function [f] = IsoMito_dXdT(t,x,~)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   FUNCTION "IsoMito_dXdT" computes the RHS derivatives of the ODE model of oxidative
%   phosphorylation and cation handling in isolated cardiac mitochondria
%   presented in the Tewari et al 2014 (J Physiol 592.9 pp 19171930)
%
% INPUTS:
%   t - time 
%   x - vector of state variables
%
% OUTPUTS: 
%   f - vector of time derivatives of concentrations
%
% The notations are based on the model of Dash-Beard (JPhysiol, 2008). In the
% present model, the kinetics of reactant and cation binding are considered.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% These global variables are for passing between the main and dXdt routines
global X_CU K_sMg X_RTC KCa_RTC
global BPT_1 KdBP_1 BPT_2 KdBP_2 
global MV_Fact Cae_add t_add

%% Concentration state variables for which differential equations will be defined
dPsi = x(1); NADH_x = x(2); QH2 = x(3); Cred = x(4); ATP_x = x(5); ADP_x = x(6); 
Pi_x = x(7); H_x = x(8); K_x = x(9); Mg_x = x(10); Ca_x = x(11); Na_x = x(12); 
ATP_i = x(13); ADP_i = x(14); Pi_i = x(15); ATP_e = x(16); ADP_e = x(17); 
Pi_e = x(18); H_e = x(19); K_e = x(20); Mg_e = x(21); Ca_e = x(22); Na_e = x(23);
% CaHPO4s_x = x(24); Ca3PO42s_x = x(25); RO = x(26); 
CaSBP1_e = x(24); Pc1 = x(25); Po1 = x(26); Po2 = x(27);

%% Anatomical parameters (mitochondrial volume, surface area, etc...)
% Mitocondrial preparation of Age etal contained 0.5 mg mito protein 
% per ml of buffer solution. Thus, W_e = 3.7e-3/(0.5*MV_Fact).
MV_Fact = 1e-6;              % 1 mg protein = 3.7e-6 Liter mito protien
W_e = 1e-3/(0.5*MV_Fact);    % external water volume (ml water / ml mito)
W_m = 0.143/0.2;             % mitochondrial water space (ml water / ml mito)
W_i = 0.1*W_m;               % IMS water space (ml water / ml mito)
W_x = 0.9*W_m;               % matrix water space (ml water / ml mito) 

%% IMS concentrations of cations: H+, K+, Mg2+, Ca2+, and Na+
pH_x = -log10(H_x);          % matrix pH
pH_e = -log10(H_e);          % buffer pH
pH_i = pH_e;                 % IMS pH
H_i = 10^(-pH_i);            % IMS [H+], M
K_i = K_e;                   % IMS [K+], M
Mg_i = Mg_e;                 % IMS [Mg2+], M
Ca_i = Ca_e;                 % IMS [Ca2+], M
Na_i = Na_e;                 % IMS [Na+], M

%% Buffer, IMS and matrix concentrations of EGTA and Ligand
EGTA_e = 0e-3;               % external EGTA concentration for cation buffering, M
EGTA_i = 0e-6;               % IMS EGTA concentration for cation buffering, M
EGTA_x = 0e-6;               % matrix EGTA concentration for cation buffering, M
Lig_e = 0e-3;                % external ligand concentration for cation buffering, M
Lig_i = 0e-3;                % IMS ligand concentration for cation buffering, M 
Lig_x = 1*(17.92e-9)/MV_Fact;      % matrix ligand concentration for cation buffering, M

%% Calcium addition is assumed guassian in nature
dt_add = 0.05;     
JCae_add = Cae_add*(1/(dt_add*sqrt(2*pi)))*exp(-(t-t_add)^2/(2*dt_add^2));
%% Proton buffering protein concentrations and binding constants in buffer and matrix
Kd_HBP_e = 10^-7.5;         % binding constant for buffer proton buffering, M (pK = 7.5)
HBPtot_e = 1*20e-3;           % buffer proton buffering protein concentration, M
Kd_HBP_x = 10^-7.5;         % binding constant for matrix proton buffering, M (pK = 7.5)
HBPtot_x = 1*20e-3;           % matrix proton buffering protein concentration, M
Kd_MgBP_x = 5.9e-6;%10^-5.23;       % binding constant for matrix Mg2+ buffering, M (pK = 5.29)
MgBPtot_x = 1*(5.58e-9)/MV_Fact;%     % matrix Mg2+ buffering protein concentration, M
Kd_CaBP_x = 1.9e-6*KdBP_1;%   % binding constant for matrix Ca2+ buffering, M (pK = 5.23)
CaBPtot_x = 35e-3*BPT_1;%     % matrix Ca2+ buffering protein concentration, M
Kd_mCaBP_x = 1.7e-6*KdBP_2;%     % binding constant for matrix Ca2+ buffering, M (pK = 5.23)
mCaBPtot_x = 15e-3*BPT_2;%      % matrix Ca2+ buffering protein concentration, M
n_mCaFBP1 = 6;

HBPbnd_e = HBPtot_e*H_e/(Kd_HBP_e + H_e);            % buffer protein-bound [H+], M
HBPbnd_H_e = HBPtot_e*Kd_HBP_e/(Kd_HBP_e + H_e)^2;   % buffer [H+]*beta_H
HBPbnd_x = HBPtot_x*H_x/(Kd_HBP_x + H_x);            % matrix protein-bound [H+], M
HBPbnd_H_x = HBPtot_x*Kd_HBP_x/(Kd_HBP_x + H_x)^2;   % matrix [H+]*beta_H
MgBPbnd_x = MgBPtot_x*Mg_x/(Kd_MgBP_x + Mg_x);       % matrix protein-bound [Mg2+], M
MgBPbnd_Mg_x = MgBPtot_x*Kd_MgBP_x/(Kd_MgBP_x + Mg_x)^2; % matrix [Mg2+]*beta_Mg
CaBPbnd_x = CaBPtot_x*Ca_x/(Kd_CaBP_x + Ca_x);           % matrix protein-bound [Ca2+], M
CaBPbnd_Ca_x = CaBPtot_x*Kd_CaBP_x/(Kd_CaBP_x + Ca_x)^2; % matrix [Ca2+]*beta_Ca
% mCaBPbnd_x = mCaBPtot_x*Ca_x/(mKd_CaBP_x + Ca_x);           % matrix protein-bound [Ca2+], M
mCaBPbnd_Ca_x = n_mCaFBP1*mCaBPtot_x*n_mCaFBP1*Kd_mCaBP_x^n_mCaFBP1*Ca_x^(n_mCaFBP1-1)/(Kd_mCaBP_x^n_mCaFBP1 + Ca_x^n_mCaFBP1)^2;% 

%% Call "get_binding_poly_frac_pd" to compute binding fractions and partial derivatives
% Calculations w.r.t. matrix space
React_tot_x(1) = ATP_x; React_tot_x(2) = ADP_x; React_tot_x(3) = Pi_x;
React_tot_x(4) = EGTA_x; React_tot_x(5) = Lig_x; 

Cation_free_x(1) = K_x; Cation_free_x(2) = Mg_x;
Cation_free_x(3) = Ca_x; Cation_free_x(4) = Na_x;

[P_React_x, React_frac_x, Cation_frac_x, ReactCation_pd_x] = ...
    get_binding_poly_frac_pd(React_tot_x, Cation_free_x, pH_x);

P_ATP_x = P_React_x(1); P_ADP_x = P_React_x(2); P_Pi_x = P_React_x(3);
P_EGTA_x = P_React_x(4); P_Lig_x = P_React_x(5); 

ATP_frac_x = React_frac_x(1,:); ADP_frac_x = React_frac_x(2,:);
Pi_frac_x = React_frac_x(3,:); EGTA_frac_x = React_frac_x(4,:);
Lig_frac_x = React_frac_x(5,:); 

H_frac_x = Cation_frac_x(1,:); K_frac_x = Cation_frac_x(2,:);
Mg_frac_x = Cation_frac_x(3,:); Ca_frac_x = Cation_frac_x(4,:);
Na_frac_x = Cation_frac_x(5,:); H_frac_x = [H_frac_x,HBPbnd_x];
Mg_frac_x = [Mg_frac_x,MgBPbnd_x]; Ca_frac_x = [Ca_frac_x,CaBPbnd_x];

HReact_pd_x = ReactCation_pd_x(1,:); KReact_pd_x = ReactCation_pd_x(2,:);
MgReact_pd_x = ReactCation_pd_x(3,:); CaReact_pd_x = ReactCation_pd_x(4,:);
NaReact_pd_x = ReactCation_pd_x(5,:);

% Calculations w.r.t. IMS
React_tot_i(1) = ATP_i; React_tot_i(2) = ADP_i; React_tot_i(3) = Pi_i;
React_tot_i(4) = EGTA_i; React_tot_i(5) = Lig_i; 

Cation_free_i(1) = K_i; Cation_free_i(2) = Mg_i;
Cation_free_i(3) = Ca_i; Cation_free_i(4) = Na_i;

[P_React_i, React_frac_i, Cation_frac_i] = ...
    get_binding_poly_frac(React_tot_i, Cation_free_i, pH_i);

P_ATP_i = P_React_i(1); P_ADP_i = P_React_i(2); P_Pi_i = P_React_i(3);
P_EGTA_i = P_React_i(4); P_Lig_i = P_React_i(5); 

ATP_frac_i = React_frac_i(1,:); ADP_frac_i = React_frac_i(2,:);
Pi_frac_i = React_frac_i(3,:); EGTA_frac_i = React_frac_i(4,:);
Lig_frac_i = React_frac_i(5,:); 

H_frac_i = Cation_frac_i(1,:); K_frac_i = Cation_frac_i(2,:);
Mg_frac_i = Cation_frac_i(3,:); Ca_frac_i = Cation_frac_i(4,:);
Na_frac_i = Cation_frac_i(5,:);

% Calculation w.r.t. external space
React_tot_e(1) = ATP_e; React_tot_e(2) = ADP_e; React_tot_e(3) = Pi_e;
React_tot_e(4) = EGTA_e; React_tot_e(5) = Lig_e; 

Cation_free_e(1) = K_e; Cation_free_e(2) = Mg_e;
Cation_free_e(3) = Ca_e; Cation_free_e(4) = Na_e;

[P_React_e, React_frac_e, Cation_frac_e, ReactCation_pd_e] = ...
    get_binding_poly_frac_pd(React_tot_e, Cation_free_e, pH_e);

P_ATP_e = P_React_e(1); P_ADP_e = P_React_e(2); P_Pi_e = P_React_e(3);
P_EGTA_e = P_React_e(4); P_Lig_e = P_React_e(5); 

ATP_frac_e = React_frac_e(1,:); ADP_frac_e = React_frac_e(2,:);
Pi_frac_e = React_frac_e(3,:); EGTA_frac_e = React_frac_e(4,:);
Lig_frac_e = React_frac_e(5,:); 

H_frac_e = Cation_frac_e(1,:); K_frac_e = Cation_frac_e(2,:);
Mg_frac_e = Cation_frac_e(3,:); Ca_frac_e = Cation_frac_e(4,:);
Na_frac_e = Cation_frac_e(5,:); H_frac_e = [H_frac_e,HBPbnd_e];

HReact_pd_e = ReactCation_pd_e(1,:); KReact_pd_e = ReactCation_pd_e(2,:);
MgReact_pd_e = ReactCation_pd_e(3,:); CaReact_pd_e = ReactCation_pd_e(4,:);
NaReact_pd_e = ReactCation_pd_e(5,:);

%% OxPhos Model Fluxes:
C_Species = zeros(23,1);
C_Species(1) = dPsi; C_Species(2) = NADH_x; C_Species(3) = QH2;
C_Species(4) = Cred; C_Species(5) = ATP_x; C_Species(6) = ADP_x;
C_Species(7) = Pi_x; C_Species(8) = ATP_i; C_Species(9) = ADP_i;
C_Species(10) = Pi_i; C_Species(11) = ATP_e; C_Species(12) = ADP_e;
C_Species(13) = Pi_e; C_Species(14) = H_x; C_Species(15) = H_i;
C_Species(16) = K_x; C_Species(17) = K_i; 
C_Species(18) = ATP_frac_x(1); C_Species(19) = ATP_frac_i(1); 
C_Species(20) = ADP_frac_x(1); C_Species(21) = ADP_frac_i(1); 
C_Species(22) = Pi_frac_x(2); C_Species(23) = Pi_frac_i(2);
P_Metab_x(1) = P_ATP_x; P_Metab_x(2) = P_ADP_x; P_Metab_x(3) = P_Pi_x;

% OxPhos Model (Beard 2005 PLoS Comput Biol 1: e36.)
[J_OxPhos] = OxPhos_model(C_Species,P_Metab_x);

J_DH = J_OxPhos(1); J_C1 = J_OxPhos(2); J_C3 = J_OxPhos(3); J_C4 = J_OxPhos(4); 
J_F1F0 = J_OxPhos(5); J_ANT = J_OxPhos(6); J_PHT = J_OxPhos(7); 
J_ATP = J_OxPhos(8); J_ADP = J_OxPhos(9); J_Pi = J_OxPhos(10); 
J_Hleak = J_OxPhos(11); J_Kleak = J_OxPhos(12); J_KHE = J_OxPhos(13);

%% Cation Transporter Fluxes:
% Ca2+ influx via Ca2+ uniporter: Ca2+ Uniporter Model (Pradhan etal, BiophysJ, 2011)
Model_index = 2;
P_Fact_CU = [X_CU 1.0 K_sMg 1];  % Factors for k_i_0, K_Ce_0, K_M_0, K_Pi, gamma,K_Cx_0 
J_CU = CU_model(Ca_x,Ca_e,Mg_x,Mg_e,Pi_x,Pi_e,dPsi*1e-3,P_Fact_CU,Model_index);

% Rate of Ca2+ influx into mitochondria via the steady-state RyR
% Modified Keizer-Levine model presented in Tewari etal 2014 (J Physiol)
RyR_Fact = [X_RTC KCa_RTC];
[dx1,dx2,dx3,J_RTC] = mRTC_model(Ca_x*1e6,Ca_e*1e6,dPsi*1e-3,Mg_e,Pc1,Po1,Po2,RyR_Fact); % unit: M s^-1

% Na+/Ca2+ Exchanger Flux Basded on a New Model of NCE (Pradhan etal, BiophysJ, 2010)
Model_index = 2; n_NCE = 3;
X_NCE = 3.0e4; KNa_NCE = 1.5; KCa_NCE = 50.0; KH1_NCE = 1.0; KH2_NCE = 1.0; mH_NCE = 0.5;
NCE_Fact = [X_NCE KNa_NCE KCa_NCE KH1_NCE KH2_NCE mH_NCE];
if (n_NCE == 2)
    [J_NCE] = NCE21_model(Na_e,Na_x,Ca_e,Ca_x,H_e,H_x,dPsi*1e-3,Model_index,NCE_Fact);
elseif (n_NCE == 3)
    [J_NCE] = NCE31_model(Na_e,Na_x,Ca_e,Ca_x,H_e,H_x,dPsi*1e-3,Model_index,NCE_Fact);
end

% Na+/H+ Exchanger Flux Basded on Dash-Beard NHE Model (JPhysiol, 2008)
Model_index = 2;
X_NHE = 1.0; KNa_NHE = 1.0; KHreg_NHE = 1.0;
NHE_Fact = [X_NHE KNa_NHE KHreg_NHE];
[J_NHE] = NHE_model(Na_e,Na_x,H_e,H_x,Model_index,NHE_Fact);

% Ca2+/H+ Exchanger Flux Based on A Proposed Electroneutral-CHE Model
Model_index = 1;
X_CHE = 0.2; KCa1_CHE = 1.0; KCa2_CHE = 1.0;
CHE_Fact = [X_CHE KCa1_CHE KCa2_CHE];
[J_CHE] = CHE_model(Ca_e,Ca_x,H_e,H_x,Model_index,CHE_Fact);

%% Ca2+ Buffering Fluxes (Ca_SBProtein, and Ca_FBProtein) 
% Ca2+ buffering in the EM with slow binding proteins
[J_CaSBP1_e] = CaeBuffering_Model(Ca_e,CaSBP1_e,Mg_e,H_e);

%% Derivatives: RHS of differential equations of the OxPhos and cation handling model
m_NCE = 3.0;        % proton stoichiometry for NCE
n_F1F0 = 3.0;       % proton stoichiometry for F1F0-ATPase
ddPsidt = (1.48e5)*(+4*J_C1 + 2*J_C3 + 4*J_C4 - n_F1F0*J_F1F0 - J_ANT - J_Hleak ...
    - J_Kleak - 4*J_CU - 2*J_RTC - (n_NCE-2)*J_NCE + 0*m_NCE*J_NCE);    % ddPsi/dt
dNADHdt = (+J_DH - J_C1)/W_x;              % dNADH/dt
dQH2dt = (+J_C1 - J_C3)/W_x;               % dQH2/dt
dCreddt = (+2*J_C3 - 2*J_C4)/W_i;          % dCred/dt
dATPdt_x = (+J_F1F0 - J_ANT)/W_x;          % dATP_x/dt
dADPdt_x = (-J_F1F0 + J_ANT)/W_x;          % dADP_x/dt
dPidt_x = (-J_F1F0 + J_PHT)/W_x;   % dPi_x/dt
dATPdt_i = (+J_ATP + J_ANT)/W_i;           % dATP_i/dt
dADPdt_i = (+J_ADP - J_ANT)/W_i;           % dADP_i/dt
dPidt_i = (-J_PHT + J_Pi)/W_i;             % dPi_i/dt
dATPdt_e = (-J_ATP)/W_e;                   % dATP_e/dt
dADPdt_e = (-J_ADP)/W_e;                   % dADP_e/dt
dPidt_e = (-J_Pi)/W_e;                     % dPi_e/dt
dCaSBP1dt_e = (+J_CaSBP1_e);               % dCaSBP1_e/dt

%% Derivation of differential equations for cations (H+, K+, Mg2+, Ca2+, Na+) in matrix
fun_deri_matrix()

%% Derivation of differential equations for cations (H+, K+, Mg2+, Ca2+, Na+) in buffer
fun_deri_buffer()

%% RHS of differential equations of the integrated OxPhos and cation handling model
f = zeros(length(x),1);            % f must be a column vector 
f(1) = ddPsidt; f(2) = dNADHdt; f(3) = dQH2dt; f(4) = dCreddt; 
f(5) = dATPdt_x; f(6) = dADPdt_x; f(7) = dPidt_x; f(8) = dHdt_x;
f(9) = dKdt_x; f(10) = dMgdt_x; f(11) = dCadt_x; f(12) = dNadt_x;
f(13) = dATPdt_i; f(14) = dADPdt_i; f(15) = dPidt_i; f(16) = dATPdt_e;  
f(17) = dADPdt_e; f(18) = dPidt_e; f(19) = dHdt_e; f(20) = dKdt_e; 
f(21) = dMgdt_e; f(22) = dCadt_e; f(23) = dNadt_e; f(24) = dCaSBP1dt_e; 
f(25) = dx1; f(26) = dx2; f(27) = dx3;

%%
function fun_deri_matrix()
% H-bound and its partial derivatives w.r.t. cations and reactants
HReact_H_x = HReact_pd_x(1); HReact_K_x = HReact_pd_x(2); 
HReact_Mg_x = HReact_pd_x(3); HReact_Ca_x = HReact_pd_x(4);
HReact_Na_x = HReact_pd_x(5); HReact_ATP_x = HReact_pd_x(6);
HReact_ADP_x = HReact_pd_x(7); HReact_Pi_x = HReact_pd_x(8);
HReact_EGTA_x = HReact_pd_x(9); HReact_Lig_x = HReact_pd_x(10);

% K-bound and its partial derivatives w.r.t. cations and reactants
KReact_H_x = KReact_pd_x(1); KReact_K_x = KReact_pd_x(2); 
KReact_Mg_x = KReact_pd_x(3); KReact_Ca_x = KReact_pd_x(4);
KReact_Na_x = KReact_pd_x(5); KReact_ATP_x = KReact_pd_x(6);
KReact_ADP_x = KReact_pd_x(7); KReact_Pi_x = KReact_pd_x(8);
KReact_EGTA_x = KReact_pd_x(9); KReact_Lig_x = KReact_pd_x(10);

% Mg-bound and its partial derivatives w.r.t. cations and reactants
MgReact_H_x = MgReact_pd_x(1); MgReact_K_x = MgReact_pd_x(2); 
MgReact_Mg_x = MgReact_pd_x(3); MgReact_Ca_x = MgReact_pd_x(4);
MgReact_Na_x = MgReact_pd_x(5); MgReact_ATP_x = MgReact_pd_x(6);
MgReact_ADP_x = MgReact_pd_x(7); MgReact_Pi_x = MgReact_pd_x(8);
MgReact_EGTA_x = MgReact_pd_x(9); MgReact_Lig_x = MgReact_pd_x(10);

% Ca-bound and its partial derivatives w.r.t. cations and reactants
CaReact_H_x = CaReact_pd_x(1); CaReact_K_x = CaReact_pd_x(2); 
CaReact_Mg_x = CaReact_pd_x(3); CaReact_Ca_x = CaReact_pd_x(4);
CaReact_Na_x = CaReact_pd_x(5); CaReact_ATP_x = CaReact_pd_x(6);
CaReact_ADP_x = CaReact_pd_x(7); CaReact_Pi_x = CaReact_pd_x(8);
CaReact_EGTA_x = CaReact_pd_x(9); CaReact_Lig_x = CaReact_pd_x(10);

% Na-bound and its partial derivatives w.r.t. cations and reactants
NaReact_H_x = NaReact_pd_x(1); NaReact_K_x = NaReact_pd_x(2); 
NaReact_Mg_x = NaReact_pd_x(3); NaReact_Ca_x = NaReact_pd_x(4);
NaReact_Na_x = NaReact_pd_x(5); NaReact_ATP_x = NaReact_pd_x(6);
NaReact_ADP_x = NaReact_pd_x(7); NaReact_Pi_x = NaReact_pd_x(8);
NaReact_EGTA_x = NaReact_pd_x(9); NaReact_Lig_x = NaReact_pd_x(10);

% Htot equation: RHS and the coefficients of derivatives of cations
Htot_dHdt_x = 1 + HReact_H_x + HBPbnd_H_x;
Htot_dKdt_x = HReact_K_x;
Htot_dMgdt_x = HReact_Mg_x;
Htot_dCadt_x = HReact_Ca_x;
Htot_dNadt_x = HReact_Na_x;
Htot_RHS_x = -(HReact_ATP_x*dATPdt_x + HReact_ADP_x*dADPdt_x ...
    + HReact_Pi_x*dPidt_x) + (J_DH - (4+1)*J_C1 - (4-2)*J_C3 - (2+2)*J_C4 ...
    + (n_F1F0-1)*J_F1F0 + 2*J_PHT + J_Hleak + J_KHE - 0*m_NCE*J_NCE + J_NHE ...
    + 2*J_CHE)/W_x;

% Ktot equation: RHS and the coefficients of derivatives of cations
Ktot_dHdt_x = KReact_H_x;
Ktot_dKdt_x = 1 + KReact_K_x;
Ktot_dMgdt_x = KReact_Mg_x;
Ktot_dCadt_x = KReact_Ca_x;
Ktot_dNadt_x = KReact_Na_x;
Ktot_RHS_x = -(KReact_ATP_x*dATPdt_x + KReact_ADP_x*dADPdt_x ...
    + KReact_Pi_x*dPidt_x) + (-J_KHE + J_Kleak)/W_x;

% Mgtot equation: RHS and the coefficients of derivatives of cations
Mgtot_dHdt_x = MgReact_H_x;
Mgtot_dKdt_x = MgReact_K_x;
Mgtot_dMgdt_x = 1 + MgReact_Mg_x + MgBPbnd_Mg_x;
Mgtot_dCadt_x = MgReact_Ca_x;
Mgtot_dNadt_x = MgReact_Na_x;
Mgtot_RHS_x = -(MgReact_ATP_x*dATPdt_x + MgReact_ADP_x*dADPdt_x ...
    + MgReact_Pi_x*dPidt_x);

% Catot equation: RHS and the coefficients of derivatives of cations
Catot_dHdt_x = CaReact_H_x;
Catot_dKdt_x  = CaReact_K_x;
Catot_dMgdt_x = CaReact_Mg_x;
Catot_dCadt_x = 1 + CaReact_Ca_x + CaBPbnd_Ca_x + mCaBPbnd_Ca_x;
Catot_dNadt_x = CaReact_Na_x;
Catot_RHS_x = -(CaReact_ATP_x*dATPdt_x + CaReact_ADP_x*dADPdt_x ...
    + CaReact_Pi_x*dPidt_x) + (2*J_CU + J_RTC - J_NCE - J_CHE)/W_x;

% Natot equation: RHS and the coefficients of derivatives of cations
Natot_dHdt_x = NaReact_H_x;
Natot_dKdt_x = NaReact_K_x;
Natot_dMgdt_x = NaReact_Mg_x;
Natot_dCadt_x = NaReact_Ca_x;
Natot_dNadt_x = 1 + NaReact_Na_x;
Natot_RHS_x = -(NaReact_ATP_x*dATPdt_x + NaReact_ADP_x*dADPdt_x ...
    + NaReact_Pi_x*dPidt_x) + (n_NCE*J_NCE - J_NHE)/W_x;

% Use Cramer's rule to solve for derivatives of cations
Coeff_dHdt_x = [Htot_dHdt_x, Ktot_dHdt_x, Mgtot_dHdt_x, Catot_dHdt_x, Natot_dHdt_x]';
Coeff_dKdt_x = [Htot_dKdt_x, Ktot_dKdt_x, Mgtot_dKdt_x, Catot_dKdt_x, Natot_dKdt_x]';
Coeff_dMgdt_x = [Htot_dMgdt_x, Ktot_dMgdt_x, Mgtot_dMgdt_x, Catot_dMgdt_x, Natot_dMgdt_x]';
Coeff_dCadt_x = [Htot_dCadt_x, Ktot_dCadt_x, Mgtot_dCadt_x, Catot_dCadt_x, Natot_dCadt_x]';
Coeff_dNadt_x = [Htot_dNadt_x, Ktot_dNadt_x, Mgtot_dNadt_x, Catot_dNadt_x, Natot_dNadt_x]';
Coeff_RHS_x = [Htot_RHS_x, Ktot_RHS_x, Mgtot_RHS_x, Catot_RHS_x, Natot_RHS_x]';

[dHdt_x, dKdt_x, dMgdt_x, dCadt_x, dNadt_x] = cram_deri(Coeff_RHS_x,Coeff_dHdt_x, Coeff_dKdt_x, Coeff_dMgdt_x, Coeff_dCadt_x, Coeff_dNadt_x);
end
function fun_deri_buffer()
        % H-bound and its partial derivatives w.r.t. cations and reactants
HReact_H_e = HReact_pd_e(1); HReact_K_e = HReact_pd_e(2); 
HReact_Mg_e = HReact_pd_e(3); HReact_Ca_e = HReact_pd_e(4);
HReact_Na_e = HReact_pd_e(5); HReact_ATP_e = HReact_pd_e(6);
HReact_ADP_e = HReact_pd_e(7); HReact_Pi_e = HReact_pd_e(8);
HReact_EGTA_e = HReact_pd_e(9); HReact_Lig_e = HReact_pd_e(10);

% K-bound and its partial derivatives w.r.t. cations and reactants
KReact_H_e = KReact_pd_e(1); KReact_K_e = KReact_pd_e(2); 
KReact_Mg_e = KReact_pd_e(3); KReact_Ca_e = KReact_pd_e(4);
KReact_Na_e = KReact_pd_e(5); KReact_ATP_e = KReact_pd_e(6);
KReact_ADP_e = KReact_pd_e(7); KReact_Pi_e = KReact_pd_e(8);
KReact_EGTA_e = KReact_pd_e(9); KReact_Lig_e = KReact_pd_e(10);

% Mg-bound and its partial derivatives w.r.t. cations and reactants
MgReact_H_e = MgReact_pd_e(1); MgReact_K_e = MgReact_pd_e(2); 
MgReact_Mg_e = MgReact_pd_e(3); MgReact_Ca_e = MgReact_pd_e(4);
MgReact_Na_e = MgReact_pd_e(5); MgReact_ATP_e = MgReact_pd_e(6);
MgReact_ADP_e = MgReact_pd_e(7); MgReact_Pi_e = MgReact_pd_e(8);
MgReact_EGTA_e = MgReact_pd_e(9); MgReact_Lig_e = MgReact_pd_e(10);

% Ca-bound and its partial derivatives w.r.t. cations and reactants
CaReact_H_e = CaReact_pd_e(1); CaReact_K_e = CaReact_pd_e(2); 
CaReact_Mg_e = CaReact_pd_e(3); CaReact_Ca_e = CaReact_pd_e(4);
CaReact_Na_e = CaReact_pd_e(5); CaReact_ATP_e = CaReact_pd_e(6);
CaReact_ADP_e = CaReact_pd_e(7); CaReact_Pi_e = CaReact_pd_e(8);
CaReact_EGTA_e = CaReact_pd_e(9); CaReact_Lig_e = CaReact_pd_e(10);

% Na-bound and its partial derivatives w.r.t. cations and reactants
NaReact_H_e = NaReact_pd_e(1); NaReact_K_e = NaReact_pd_e(2); 
NaReact_Mg_e = NaReact_pd_e(3); NaReact_Ca_e = NaReact_pd_e(4);
NaReact_Na_e = NaReact_pd_e(5); NaReact_ATP_e = NaReact_pd_e(6);
NaReact_ADP_e = NaReact_pd_e(7); NaReact_Pi_e = NaReact_pd_e(8);
NaReact_EGTA_e = NaReact_pd_e(9); NaReact_Lig_e = NaReact_pd_e(10);

% Htot equation: RHS and the coefficients of derivatives of cations
Htot_dHdt_e = 1 + HReact_H_e + HBPbnd_H_e;
Htot_dKdt_e = HReact_K_e;
Htot_dMgdt_e = HReact_Mg_e;
Htot_dCadt_e = HReact_Ca_e;
Htot_dNadt_e = HReact_Na_e;
Htot_RHS_e = -(HReact_ATP_e*dATPdt_e + HReact_ADP_e*dADPdt_e ...
    + HReact_Pi_e*dPidt_e) - (0*J_DH - (4+0)*J_C1 - (4-0)*J_C3 ...
    - (2+0)*J_C4 + (n_F1F0-0)*J_F1F0 + 2*J_PHT + J_Hleak ...
    + J_KHE + 0*m_NCE*J_NCE + J_NHE + 2*J_CHE)/W_e;

% Ktot equation: RHS and the coefficients of derivatives of cations
Ktot_dHdt_e = KReact_H_e;
Ktot_dKdt_e = 1 + KReact_K_e;
Ktot_dMgdt_e = KReact_Mg_e;
Ktot_dCadt_e = KReact_Ca_e;
Ktot_dNadt_e = KReact_Na_e;
Ktot_RHS_e = -(KReact_ATP_e*dATPdt_e + KReact_ADP_e*dADPdt_e ...
    + KReact_Pi_e*dPidt_e) - (-J_KHE + J_Kleak)/W_e;

% Mgtot equation: RHS and the coefficients of derivatives of cations
Mgtot_dHdt_e = MgReact_H_e;
Mgtot_dKdt_e = MgReact_K_e;
Mgtot_dMgdt_e = 1 + MgReact_Mg_e;
Mgtot_dCadt_e = MgReact_Ca_e;
Mgtot_dNadt_e = MgReact_Na_e;
Mgtot_RHS_e = -(MgReact_ATP_e*dATPdt_e + MgReact_ADP_e*dADPdt_e ...
    + MgReact_Pi_e*dPidt_e);

% Catot equation: RHS and the coefficients of derivatives of cations
Catot_dHdt_e = CaReact_H_e;
Catot_dKdt_e = CaReact_K_e;
Catot_dMgdt_e = CaReact_Mg_e;
Catot_dCadt_e = 1 + CaReact_Ca_e;
Catot_dNadt_e = CaReact_Na_e;
Catot_RHS_e = -(CaReact_ATP_e*dATPdt_e + CaReact_ADP_e*dADPdt_e ...
    + CaReact_Pi_e*dPidt_e  + 1*dCaSBP1dt_e  - JCae_add) - (2*J_CU + J_RTC - J_NCE - J_CHE)/W_e;

% Natot equation: RHS and the coefficients of derivatives of cations
Natot_dHdt_e = NaReact_H_e;
Natot_dKdt_e = NaReact_K_e;
Natot_dMgdt_e = NaReact_Mg_e;
Natot_dCadt_e = NaReact_Ca_e;
Natot_dNadt_e = 1 + NaReact_Na_e;
Natot_RHS_e = -(NaReact_ATP_e*dATPdt_e + NaReact_ADP_e*dADPdt_e ...
    + NaReact_Pi_e*dPidt_e) - (n_NCE*J_NCE - J_NHE)/W_e;

% Use Cramer's rule to solve for derivatives of cations
Coeff_dHdt_e = [Htot_dHdt_e, Ktot_dHdt_e, Mgtot_dHdt_e, Catot_dHdt_e, Natot_dHdt_e]';
Coeff_dKdt_e = [Htot_dKdt_e, Ktot_dKdt_e, Mgtot_dKdt_e, Catot_dKdt_e, Natot_dKdt_e]';
Coeff_dMgdt_e = [Htot_dMgdt_e, Ktot_dMgdt_e, Mgtot_dMgdt_e, Catot_dMgdt_e, Natot_dMgdt_e]';
Coeff_dCadt_e = [Htot_dCadt_e, Ktot_dCadt_e, Mgtot_dCadt_e, Catot_dCadt_e, Natot_dCadt_e]';
Coeff_dNadt_e = [Htot_dNadt_e, Ktot_dNadt_e, Mgtot_dNadt_e, Catot_dNadt_e, Natot_dNadt_e]';
Coeff_RHS_e = [Htot_RHS_e, Ktot_RHS_e, Mgtot_RHS_e, Catot_RHS_e, Natot_RHS_e]';
[dHdt_e, dKdt_e, dMgdt_e, dCadt_e, dNadt_e] = cram_deri(Coeff_RHS_e,Coeff_dHdt_e, Coeff_dKdt_e, Coeff_dMgdt_e, Coeff_dCadt_e, Coeff_dNadt_e);
    end
end