% Kinetic model of the mitochondrial Ca2+/2H+ exchanger (CHE) that is derived to 
% qualitatively fit to the experimental data of Wingrove etal (1986) and Coll 
% etal (1982) obtained using isolated mitochondria from rat liver and rat heart.
% Ca_x: Internal calcium ion concentration (M)
% Ca_e: External calcium ion concentration (M)
% H_x: Internal hydrogen ion concentration (M)
% H_e: External hydrogen ion concentration (M)

function [CHE_flux] = CHE_model(Ca_e,Ca_x,H_e,H_x,mod_index,par_fact)

global MV_Fact % Mito volume conversion factor for activity: mol/mg/s --> nmol/mg/s or mol/L/s

if (mod_index == 1) % Hyperbolic CHE model (nH = 1) to fit to the data of Coll et al (1982)
    CHE_param = [4.7e-9 % Vmax of CHE; Apparent Vmax = 4.7 nmol/mg/min (Coll et al, 1982)
                 4.8e-6]; % Km of Ca for CHE (M); Apparent Km = 9.7 uM (Coll et al, 1982)
    Vmax =  CHE_param(1)*par_fact(1)/(60*MV_Fact); % nmol/mg/s or mol/L/s
    Km = CHE_param(2)*par_fact(2); % molar
    CHE_flux = Vmax * ((H_e^2*Ca_x - H_x^2*Ca_e) / ...
        (Km*(H_e^2+H_x^2) + H_e^2*Ca_x + H_x^2*Ca_e));
elseif (mod_index == 2) % Sigmoid CHE model (nH = 2) to fit to the data of Wingrove et al (1986)
    CHE_param = [1.2e-9 % Vmax of CHE; Apparent Vmax = 1.2 nmol/mg/min (Wingrove et al, 1986)
                 3.0e-6 % Km1 of Ca for CHE (M); Apparent Km = 6 uM Wingrove et al, 1986)
                 6.0e-6]; % Km2 of Ca for CHE (M); Apparent Km = 6 uM Wingrove et al, 1986)
    Vmax = CHE_param(1)*par_fact(1)/(60*MV_Fact);   % nmol/mg/s or mol/L/s
    Km1 = CHE_param(2)*par_fact(2);  % molar
    Km2 = CHE_param(3)*par_fact(3);  % molar
    CHE_flux = Vmax * (Ca_x / (Km2 + Ca_x)) * ((H_e^2*Ca_x - H_x^2*Ca_e) / ...
        (Km1*(H_e^2+H_x^2) + H_e^2*Ca_x + H_x^2*Ca_e));
end
