%------------------------------------------------------------------------------------
% Final scheme: CU models with two binding sites for Ca2+ and two allosteric binding 
% sites for Mg2+ (distinct from Ca2+). This version includes Pi regulation of KC and
% KM. In this version, Mg2+ inhibits the CU activity only through external binding. 
% Developed by R.K. Pradhan @ BBC, MCW (recent revision: 02/06/2013).
%------------------------------------------------------------------------------------

function [CU_flux] = CU_model(Ca_x,Ca_e,Mg_x,Mg_e,Pi_x,Pi_e,dPsi,par_fact,mod_index)

global MV_Fact % Mito volume conversion factor for activity: mol/mg/s --> nmol/mg/s or mol/L/s

% CU model parameters as estimated
if (mod_index == 1)
   CU_param = [1.30e-12   % k_i_0 (mol/mg/sec)
        5.15e-6      % K_C_0(M)
        0.85e-3      % K_M_0(M)
        3.7];        % gamma
elseif (mod_index == 2)
    CU_param = [1.55e-12   % k_i_0 (mol/mg/sec)
        4.15e-6      % K_C_0(M)
        0.80e-3      % K_M_0(M)
        4.8];        % gamma
elseif (mod_index == 3)
    CU_param = [1.75e-12   % k_i_0 (mol/mg/sec)
        3.50e-6      % K_C_0(M)
        0.80e-3      % K_M_0(M)
        6.1];        % gamma
end

% Revise or fine tune CU model parameters (as necessary)
CU_param(1) = CU_param(1)/MV_Fact;       % Convert to nmol/mg/sec or mol/L/sec
CU_param(1) = CU_param(1)*par_fact(1);   % Modify CU Activity
CU_param(2) = CU_param(2)*par_fact(2);   % Modify Ca2+ Binding Constant
CU_param(3) = CU_param(3)*par_fact(3);   % Modify Mg2+ Binding Constant 
CU_param(4) = CU_param(4)*par_fact(4);   % Modify gamma

% Standard thermodynamic parameter values
Z_Mg = 2;                % Valency of Mg(uniteless)
Z_Ca = Z_Mg;             % Valency of Ca(uniteless)
F = 96484.6;             % Faraday's constant(J*V^-1*mol^-1)(Coul*mol^-1)
R = 8314e-3;             % Universal gas constant(J*mol^-1*K^-1)
T = 298;                 % Temperature(K)
FRT = F/(R*T);           % V^-1

% Assign appropriate model parameter values
k_i_0 = CU_param(1);     % Forward rate constant(M/sec)
K_Ce_0 = CU_param(2);    % Binding constant of external Ca2+(M)
K_Me_0 = CU_param(3);    % Binding constant of external Mg2+(M)
gamma = CU_param(4);     % Control parameter for type of Mg inhibition

% Parameters through kinetic and thermodynamic constraints
K_Pi = 0.2e-3;  % Binding constant of Pi for the uniporter(M)
K_Cx_0 = K_Ce_0;
K_Mx_0 = 1000*K_Me_0;

% Modification of K_Ce and K_Me by Pi
K_Ce_0 = K_Ce_0*(1+Pi_e/(K_Pi+Pi_e));
K_Cx_0 = K_Cx_0*(1+Pi_x/(K_Pi+Pi_x));
K_Me_0 = K_Me_0/(1+Pi_e/(K_Pi+Pi_e));
K_Mx_0 = K_Mx_0/(1+Pi_x/(K_Pi+Pi_x));

% Parameters from kinetics & thermodynamic constraints
k_o_0 = k_i_0*(K_Cx_0/K_Ce_0)^2;

% dPsi dependency of the kinetic parameters
nH = 2.65;          % Hill coefficient for dPsi dependency of beta
alpha_e = 0;        % Energy barrier parameter for K_Ce and K_Me
alpha_x = alpha_e;  % Energy barrier parameter for K_Cx and K_Mx

dPhi = Z_Ca*dPsi*FRT;
Fact = ((dPhi/nH)/sinh(dPhi/nH))^nH;
beta_e = 0.5*(1 + log(Fact)/dPhi) - alpha_e;
beta_x = 0.5*(1 - log(Fact)/dPhi) - alpha_x;

k_i = k_i_0*exp(+2*beta_e*dPhi);
k_o = k_o_0*exp(-2*beta_x*dPhi);
K_Ce = K_Ce_0*exp(-1*alpha_e*dPhi);
K_Cx = K_Cx_0*exp(+1*alpha_x*dPhi);
K_Me = K_Me_0*exp(-1*alpha_e*dPhi);
K_Mx = K_Mx_0*exp(+1*alpha_x*dPhi);

% Calcium uniporter flux expression
num = k_i*(Ca_e^2/K_Ce^2) - k_o*(Ca_x^2/K_Cx^2);

if (mod_index == 1) % Model 1
    den = 1 + (Ca_e^2/K_Ce^2) + (Ca_x^2/K_Cx^2) + (Mg_e^2/K_Me^2) + (Mg_x^2/K_Mx^2)...
        + (Ca_e^2/(gamma^2*K_Ce^2))*(Mg_e^2/(gamma^2*K_Me^2))...
        + (Ca_x^2/(gamma^2*K_Cx^2))*(Mg_x^2/(gamma^2*K_Mx^2));
elseif (mod_index == 2) % Model 2
    den = 1 + (Ca_e/K_Ce) + (Ca_x/K_Cx) + (Ca_e^2/K_Ce^2) + (Ca_x^2/K_Cx^2) ...
        + (Mg_e/K_Me) + (Mg_x/K_Mx) + (Mg_e^2/K_Me^2) + (Mg_x^2/K_Mx^2) ...
        + (Ca_e*Mg_e)/(gamma*K_Ce*K_Me) + (Ca_x*Mg_x)/(gamma*K_Cx*K_Mx) ...
        + (Ca_e^2*Mg_e)/(gamma^2*K_Ce^2*K_Me) + (Ca_x^2*Mg_x)/(gamma^2*K_Cx^2*K_Mx) ...
        + (Ca_e*Mg_e^2)/(gamma^2*K_Ce*K_Me^2) + (Ca_x*Mg_x^2)/(gamma^2*K_Cx*K_Mx^2) ...
        + (Ca_e^2*Mg_e^2)/(gamma^4*K_Ce^2*K_Me^2) + (Ca_x^2*Mg_x^2)/(gamma^4*K_Cx^2*K_Mx^2);
elseif (mod_index == 3) % Model 3
    den = 1 + 2*(Ca_e/K_Ce) + 2*(Ca_x/K_Cx) + (Ca_e^2/K_Ce^2) + (Ca_x^2/K_Cx^2) ...
        + 2*(Mg_e/K_Me) + 2*(Mg_x/K_Mx) + (Mg_e^2/K_Me^2) + (Mg_x^2/K_Mx^2) ...
        + 4*(Ca_e*Mg_e)/(gamma*K_Ce*K_Me) + 4*(Ca_x*Mg_x)/(gamma*K_Cx*K_Mx) ...
        + 2*(Ca_e^2*Mg_e)/(gamma^2*K_Ce^2*K_Me) + 2*(Ca_x^2*Mg_x)/(gamma^2*K_Cx^2*K_Mx) ...
        + 2*(Ca_e*Mg_e^2)/(gamma^2*K_Ce*K_Me^2) + 2*(Ca_x*Mg_x^2)/(gamma^2*K_Cx*K_Mx^2) ...
        + (Ca_e^2*Mg_e^2)/(gamma^4*K_Ce^2*K_Me^2) + (Ca_x^2*Mg_x^2)/(gamma^4*K_Cx^2*K_Mx^2);
end

CU_flux = num/den;
