% Kinetic model of the mitochondrial 3Na+/Ca2+ exchanger (NCE) that is derived to 
% qualitatively fit to the experimental data of Paucek and Jaburek (2004) obtained
% using NCE reconstituted in proteoliposome and Cox and Matlib (1993) obtained using 
% isolated mitochondria from rat heart.
% Ca_x: Internal calcium ion concentration (M)
% Ca_e: External calcium ion concentration (M)
% Na_x: Internal sodium ion concentration (M)
% Na_e: External sodium ion concentration (M)
% H_x: Internal hydrogen ion concentration (M)
% H_e: External hydrogen ion concentration (M)
% dPsi: Inner mitochondrial membrane potential (V)

function [NCE_flux] = NCE31_model(Na_e,Na_x,Ca_e,Ca_x,H_e,H_x,dPsi,mod_index,par_fact)

global MV_Fact % Mito volume conversion factor for activity: mol/mg/s --> nmol/mg/s or mol/L/s

pH_e = -log10(H_e); pH_x = -log10(H_x);

% NCE Model Parameters: Pradhan et al, Biophys J, 2010 (reevalauted)
if (mod_index == 1)
    NCE_param = [2.35e-15  % Forward rate constant (mol/mg/sec)
        2.1e-3        % Binding Constant of External Na+ (M)
        2.3e-9        % Binding Constant of External Ca2+ (M)
        6.4e-8        % First proton binding constants (M)
        1.4e-7];      % Second proton binding constants (M)
elseif (mod_index == 2)
    NCE_param = [2.8e-15   % Forward rate constant (mol/mg/sec)
        1.45e-3       % Binding Constant of External Na+ (M)
        2.3e-9        % Binding Constant of External Ca2+ (M)
        6.4e-8        % First proton binding constants (M)
        1.4e-7];      % Second proton binding constants (M)
elseif (mod_index == 3)
    NCE_param = [3.3e-15   % Forward rate constant (mol/mg/sec)
        1.0e-3         % Binding Constant of External Na+ (M)
        2.3e-9         % Binding Constant of External Ca2+ (M)
        6.4e-8         % First proton binding constants (M)
        1.4e-7];       % Second proton binding constants (M)
end
 
% Revise or fine tune NCE model parameters (as necessary)
NCE_param(1) = NCE_param(1)*par_fact(1)/MV_Fact;  % NCE Activity
NCE_param(2) = NCE_param(2)*par_fact(2);  % Na+ Binding Constant
NCE_param(3) = NCE_param(3)*par_fact(3);  % Ca2+ Binding Constant
NCE_param(4) = NCE_param(4)*par_fact(4);  % Ca2+ Binding Constant
NCE_param(5) = NCE_param(5)*par_fact(5);  % Ca2+ Binding Constant

% THERMODYNAMICS CONSTANTS
Z_Ca = 2;                % Valency of Ca(uniteless)
Z_Na = 1;                % Valency of Na(uniteless)
T = 298;                 % Temperature(K)
F = 96484.6;             % Faraday's constant(J * V^-1 * mol^-1) (Coul * mol^-1)
R = 8314e-3;             % Universal gas constant(J * mol^-1 * K^-1)
FRT = F/(R*T);           % V^-1 

% KINETIC PARAMETERS AND CONSTRAINTS
K_Ne_0 = NCE_param(2);   % Binding Constant of External Na+ (M)
K_Nx_0 = K_Ne_0;         % Binding constant for internal Na+ (M)
K_Ce_0 = NCE_param(3);   % Binding Constant of External Ca2+ (M)
K_Cx_0 = K_Ce_0;         % Binding constant for internal Ca2+(M)
K_H1   = NCE_param(4);   % First proton binding constants (M)
K_H2   = NCE_param(5);   % Second proton binding constants (M)
k_a_0  = NCE_param(1);   % Forward rate constant (nmol Ca2+/mg/min)
k_b_0  = k_a_0*(K_Ce_0/K_Cx_0)*(K_Nx_0/K_Ne_0)^3;  % Constraint

% BIOPHYSICAL PARAMETERS AND THERMODYNAMIC CONSTRAINTS
alpha = 0.0;             % Unitless
betta = 0.5 - alpha;     % Unitless
betta_Ne = betta;        % Unitless
betta_Nx = betta;        % Unitless
betta_Ce = betta_Ne;     % Unitless
betta_Cx = betta_Nx;     % Unitless
alpha_Ne = alpha;        % Unitless
alpha_Nx = alpha;        % Unitless
alpha_Ce = alpha_Ne;     % Unitless
alpha_Cx = alpha_Nx;     % Unitless

% pH REGULATION MECHANISM OF NCE; 6 PROTON BINDS TO NCE (m_NCE = 6) 
m_NCE = 6*par_fact(6);
H_e = 10^-pH_e;
H_x = 10^-pH_x;
sum_e1 = 0; sum_e2 = 0;
sum_x1 = 0; sum_x2 = 0;

for i = 1:m_NCE
    sum_e1 = sum_e1 + (H_e^i/K_H1^i);
    sum_e2 = sum_e2 + (K_H2^i/H_e^i);
    sum_x1 = sum_x1 + (H_x^i/K_H1^i);
    sum_x2 = sum_x2 + (K_H2^i/H_x^i);
end

D_Hee = 1 + sum_e1 + sum_e2;
D_He  = D_Hee*(H_e^m_NCE/K_H1^m_NCE);
D_Hxx = 1 + sum_x1 + sum_x2;
D_Hx  = D_Hxx*(H_x^m_NCE/K_H1^m_NCE);

K_Cx_0_app = K_Cx_0*D_Hxx;
K_Ce_0_app = K_Ce_0*D_Hee;

% NCE-FLUX CALCULATION
dPhi = dPsi*FRT;
K_Ne = K_Ne_0 * exp(-alpha_Ne*Z_Na*dPhi);
K_Nx = K_Nx_0 * exp(+alpha_Nx*Z_Na*dPhi);
K_Ce_app = K_Ce_0_app * exp(-alpha_Ce*Z_Ca*dPhi);
K_Cx_app = K_Cx_0_app * exp(+alpha_Cx*Z_Ca*dPhi);
k_a = k_a_0 * exp((+3*betta_Ne*Z_Na - betta_Cx*Z_Ca)*dPhi);
k_b = k_b_0 * exp((-3*betta_Nx*Z_Na + betta_Ce*Z_Ca)*dPhi);

NUM = (k_a*D_Hx)*(Ca_x*Na_e^3)/(K_Cx_app*K_Ne^3) ...
    - (k_b*D_He)*(Ca_e*Na_x^3)/(K_Ce_app*K_Nx^3);

if (mod_index == 1)
    DEN = D_Hx*(1+(Na_e^3/K_Ne^3)+(Ca_x/K_Cx_app)+(Ca_x*Na_e^3)/(K_Cx_app*K_Ne^3))...
        + D_He*(1+(Na_x^3/K_Nx^3)+(Ca_e/K_Ce_app)+(Ca_e*Na_x^3)/(K_Ce_app*K_Nx^3))-1;
elseif (mod_index == 2)
    DEN = D_Hx*(1+(Na_e/K_Ne)+(Na_e^2/K_Ne^2)+(Na_e^3/K_Ne^3)+(Ca_x/K_Cx_app)...
        +(Ca_x*Na_e^3)/(K_Cx_app*K_Ne^3)+(Ca_x*Na_e^2)/(K_Cx_app*K_Ne^2)...
        +(Ca_x*Na_e)/(K_Cx_app*K_Ne))...
        +D_He*(1+(Na_x/K_Nx)+(Na_x^2/K_Nx^2)+(Na_x^3/K_Nx^3)+(Ca_e/K_Ce_app)...
        +(Ca_e*Na_x^3)/(K_Ce_app*K_Nx^3)+(Ca_e*Na_x^2)/(K_Ce_app*K_Nx^2)...
        +(Ca_e*Na_x)/(K_Ce_app*K_Nx))-1;
elseif (mod_index == 3)
    DEN = D_Hx*(1+(3*Na_e/K_Ne)+(3*Na_e^2/K_Ne^2)+(Na_e^3/K_Ne^3)+(Ca_x/K_Cx_app)...
        +(Ca_x*Na_e^3)/(K_Cx_app*K_Ne^3)+(3*Ca_x*Na_e^2)/(K_Cx_app*K_Ne^2)...
        +(3*Ca_x*Na_e)/(K_Cx_app*K_Ne))...
        +D_He*(1+(3*Na_x/K_Nx)+(3*Na_x^2/K_Nx^2)+(Na_x^3/K_Nx^3)+(Ca_e/K_Ce_app)...
        +(Ca_e*Na_x^3)/(K_Ce_app*K_Nx^3)+(3*Ca_e*Na_x^2)/(K_Ce_app*K_Nx^2)...
        +(3*Ca_e*Na_x)/(K_Ce_app*K_Nx))-1;
end
NCE_flux = NUM/DEN;