% Kinetic model of the mitochondrial Na+/H+ exchanger (NHE) that is derived to 
% qualitatively fit to the experimental data of Kapus etal (1988) and Kapus 
% etal (1989) obtained from isolated mitochondria from rat liver and rat heart.
% Na_x: Internal sodium ion concentration (M)
% Na_e: External sodium ion concentration (M)
% H_x: Internal hydrogen ion concentration (M)
% H_e: External hydrogen ion concentration (M)

function [NHE_flux] = NHE_model(Na_e,Na_x,H_e,H_x,mod_index,par_fact)

global MV_Fact % Mito volume conversion factor for activity: mol/mg/s --> nmol/mg/s or mol/L/s

if (mod_index == 1)         % NHE model-1 (Dash et al, J Physiol, 2008)
    NHE_param = [18e-9      % Activity of NHE (mol/mg/s)
                 22e-3      % Binding constant of Na+ for NHE (M)
                 10^-7.0];  % Binding constant of H+ for NHE regulation by matrix pH (M)
    V_max = NHE_param(1)*par_fact(1)/MV_Fact;    % NHE activity (mol/L mito/s) 
    Km_Na = NHE_param(2)*par_fact(2);
    Km_Hreg = NHE_param(3)*par_fact(3);
    NHE_flux = V_max * (H_x/(H_x+Km_Hreg)) * (H_x/Km_Hreg) * ...
        (H_e*Na_x - H_x*Na_e) / (Km_Na*(H_e+H_x) + H_e*Na_x + H_x*Na_e);
elseif (mod_index == 2)    % NHE model-2 (Modified from Dash et al, J Physiol, 2008)
    NHE_param = [12e-9     % Activity of NHE (mol/mg/s)
                 25e-3     % Binding constant of Na+ for NHE (M)
                 10^-7.2]; % Binding constant of H+ for NHE regulation by matrix pH (M)
    n_Hreg = 3;            % Cooperativity for NHE regulation by matrix pH
    V_max = NHE_param(1)*par_fact(1)/MV_Fact;   % NHE activity (mol/L mito/s) 
    Km_Na = NHE_param(2)*par_fact(2);
    Km_Hreg = NHE_param(3)*par_fact(3);
    NHE_flux = V_max * (H_x^n_Hreg / (H_x^n_Hreg + Km_Hreg^n_Hreg)) * ...
        (H_e*Na_x - H_x*Na_e) / (Km_Na*(H_e+H_x) + H_e*Na_x + H_x*Na_e);
end