function [J_OxPhos] = OxPhos_model(C_Species,P_Metab_x)

% FUNCTION OxPhos_model computes the fluxes relating the oxidative phosphorylation.
% The fluxes are based on Dan's original model with appropriate modifications

J_OxPhos = zeros(13,1); 
dPsi = C_Species(1); NADH_x = C_Species(2); QH2 = C_Species(3); 
Cred = C_Species(4); ATP_x = C_Species(5); ADP_x = C_Species(6); 
Pi_x = C_Species(7); ATP_i = C_Species(8); ADP_i = C_Species(9); 
Pi_i = C_Species(10); ATP_e = C_Species(11); ADP_e = C_Species(12);
Pi_e = C_Species(13); H_x = C_Species(14); H_i = C_Species(15); 
K_x = C_Species(16); K_i = C_Species(17);
ATP_free_x = C_Species(18); ATP_free_i = C_Species(19);
ADP_free_x = C_Species(20); ADP_free_i = C_Species(21);
H2PO4_x = C_Species(22); H2PO4_i = C_Species(23);
P_ATP_x = P_Metab_x(1); P_ADP_x = P_Metab_x(2); P_Pi_x = P_Metab_x(3);
 
%% ETC model fluxes involved in the oxidative phosphorylation
% Standardd biochemical or thermodynamic parameters
F = 0.0964846;            % Faraday's constant (10^6 Coul*mol^{-1}=kJ*mV^{-1}*mol^{-1})
R = 8314e-6;              % universal gas constant (kJ*mol^{-1}*K^{-1})
T = 298.15;               % temperature (K)
RT = R*T;                 % gas constant times temperature (kJ mol^{-1})
dG_C1o = -69.37;          % standard free energy, complex I (kJ mol^{-1})
dG_C3o = -32.53;          % standard free energy, complex III (kJ mol^{-1})
dG_C4o = -122.94;         % standard free energy, complex IV (kJ mol^{-1})
dG_F1F0o = 36.03;         % standard free energy, F1F0-ATPase (kJ mol^{-1})
n_F1F0 = 3.0;             % proton stoichiometry for F1F0-ATPase

% Adjustable parameters obtained from optimization (Dan's revised OxPhos parameters)
k_Pi1 = 0.1553e-3;        % dehydrogenase model parameter (M)
k_Pi2 = 0.8222e-3;        % dehydrogenase model parameter (M)
k_Pi3 = 0.3601e-3;        % complex III/Pi model parameter (M)
k_Pi4 = 5.9242e-3;        % complex III/Pi model parameter (M)
k_O2 = 1.2e-4;            % Michaelis-Menton constant of O2 for complex IV (M)
r_DH = 4.559;             % dehydrogenase model parameter (unitless)
x_DH = 0.0866;            % dehydrogenase activity (mol s^{-1} M^{-1} (l mito)^{-1}) 
x_C1 = 4405;              % complex I activity (mol s^{-1} M^{-2} (l mito)^{-1})
x_C3 = 4.887;             % complex III activity (mol s^{-1} M^{-3/2} (l mito)^{-1})
x_C4 = 6.766e-5;          % complex IV activity (mol s^{-1} M^{-5/4} (l mito)^{-1})
x_F1F0 = 1000;            % F1F0-ATPase activity (mol s^{-1} M^{-1} (l mito)^{-1})

% Total concentration pools in the matrix
O2_x = 2.6e-5;            % free oxygen in matrix (pO2 = 20 mmHg, alpha = 1.3e-6 M/mmHg)
Ctot = 2.7e-3;            % total CytC (Cred+Cox), molar
Qtot = 1.35e-3;           % total Ubiquinol (Q+QH2), molar
NADtot = 3.0e-3;          % total NAD_x+NADH_x, molar
ANtot = 10.0e-3;          % total adenonucleotides (ATP+ADP), molar

% Calculations of other state variables
NAD_x = NADtot - NADH_x;
Q  =  Qtot - QH2;
Cox = Ctot - Cred;
if (ATP_x <= 1e-10)
    ATP_x = 0;
    ADP_x = ANtot;
elseif (ADP_x <= 1e-10)
    ADP_x = 0;
    ATP_x = ANtot;
end

% Membrane proton motive force and ETC apparent deltaG (matches with Fan's JBC paper).
dG_H = F*dPsi + RT*log(H_i/H_x);
dG_C1op = dG_C1o - 1*RT*log(H_x/1e-7);
dG_C3op = dG_C3o + 2*RT*log(H_x/1e-7);
dG_C4op = dG_C4o - 2*RT*log(H_x/1e-7);
dG_F1F0op = dG_F1F0o - 1*RT*log(H_x/1e-7);

% Definition of ETC fluxes (J_DH, J_C1, J_C3, and J_C4) 
r_DH = r_DH*1.6;   % Increase state 2 NADH/NAD ratio (to fit Stowe-lab data)
x_DH = x_DH*0.6;   % Increase state 3 NADH oxidation (to fit Stowe-lab data)
J_DH = x_DH*((1+Pi_x/k_Pi1)/(1+Pi_x/k_Pi2))*(r_DH*NAD_x-NADH_x);
J_C1 = x_C1*(exp(-(dG_C1op+4*dG_H)/RT)*NADH_x*Q - NAD_x*QH2);
J_C3 = x_C3*((1+Pi_x/k_Pi3)/(1+Pi_x/k_Pi4))* ...
    (exp(-(dG_C3op+4*dG_H-2*F*dPsi)/(2*RT))*Cox*QH2^0.5 - Cred*Q^0.5);
J_C4 = x_C4*(O2_x/(O2_x+k_O2))*(Cred/Ctot)* ...
    (exp(-(dG_C4op+2*dG_H)/(2*RT))*Cred*(O2_x^0.25) - Cox*exp(F*dPsi/RT));

% Definition of J_F1F0 uses binding polynomials for ATP, ADP and Pi
J_F1F0 = x_F1F0*(exp(-(dG_F1F0op-n_F1F0*dG_H)/RT)*...
    (P_ATP_x/(P_ADP_x*P_Pi_x))*ADP_x*Pi_x - ATP_x);

%% ANT model flux (Dan's original model) involved in the oxidative phosphorylation
x_ANT = 8.123e-3;   % ANT activity (mol s^{-1}(l mito)^{-1})
x_ANT = x_ANT*0.5;  % Decrease state 3 oxidation & increase state 3 duration (to fit Stowe-lab data)
k_mADP = 3.5e-6;    % ANT Michaelis-Menton constant (M)
if (ADP_free_i > 1e-10) || (ATP_free_i > 1e-10)
    J_ANT = x_ANT*(ADP_free_i/(k_mADP + ADP_free_i))* ...
        (ADP_free_i/(ADP_free_i+ATP_free_i*exp(-0.35*F*dPsi/RT)) - ...
        ADP_free_x/(ADP_free_x+ATP_free_x*exp(+0.65*F*dPsi/RT)));
else
    J_ANT = 0;
end  

%% PHT model flux (Dan's original model) involved in the oxidative phosphorylation
k_PHT = 0.2542e-3;        % H+/Pi- co-transport parameter (M)
x_PHT = 3.850e+5;         % H+/Pi- co-transport activity (mol s^{-1} M^{-1} (l mito)^{-1}) 
J_PHT = x_PHT*(H_i*H2PO4_i - H_x*H2PO4_x)/(k_PHT + H2PO4_i);

%% OMM metabolite transport fluxes (passive) involved in the oxidative phosphorylation
x_AN = 85;        % mito outer membrane permeability to adenine nucleotides (micron s^{-1})
x_Pi = 327;       % mito outer membrane permeability to inorgainic phosphate (micron s^{-1})
gamma = 5.99;     % mito outer membrane area per unit volume (micron^{-1})
J_ATP = gamma*x_AN*(ATP_e-ATP_i);
J_ADP = gamma*x_AN*(ADP_e-ADP_i);
J_Pi = gamma*x_Pi*(Pi_e-Pi_i);

%% H+ and K+ leak fluxes (simple GHK models) involved in the oxidative phosphorylation
x_Hleak = 200;    % proton leak activity (mol s^{-1} M^{-1} mV^{-1}(l mito)^{-1})
x_Kleak = 0e-2;   % passive potassium transport activity (mol s^{-1} M^{-1} mV^{-1} (l mito)^{-1})
J_Hleak = x_Hleak*dPsi*(H_i*exp(F*dPsi/RT)-H_x)/(exp(F*dPsi/RT)-1);
J_Kleak = x_Kleak*dPsi*(K_i*exp(F*dPsi/RT)-K_x)/(exp(F*dPsi/RT)-1);

%% KHE flux expression (a simple exchanger) involved in the oxidative phosphorylation
x_KHE = 5.651e7;  % K+/H+ antiporter activity (mol s^{-1} M^{-2} (l mito)^{-1})
J_KHE = x_KHE*(K_x*H_i - K_i*H_x);  % H+ going in and K+ going out

%% Assign the OXPhos fluxes to J_OxPhos to pass on to the dXdT function
J_OxPhos(1) = J_DH; J_OxPhos(2) = J_C1; J_OxPhos(3) = J_C3; J_OxPhos(4) = J_C4; 
J_OxPhos(5) = J_F1F0; J_OxPhos(6) = J_ANT; J_OxPhos(7) = J_PHT; 
J_OxPhos(8) = J_ATP; J_OxPhos(9) = J_ADP; J_OxPhos(10) = J_Pi; 
J_OxPhos(11) = J_Hleak; J_OxPhos(12) = J_Kleak; J_OxPhos(13) = J_KHE;

