function [P_React, React_frac, Cation_frac] = ...
    get_binding_poly_frac(React_tot, Cation_free, pH)

% This code computes the binding fractions of reactants (ATP, ADP, Pi, EGTA, etc.) 
% with cations (H+, K+, Mg2+, Ca2+, Na+). The inputs are total reactant concentration, 
% free cation concentration, and pH. The outputs are reactant binding polynomials
% and reactant and cation binding fractions.

ATP_tot  = React_tot(1); ADP_tot = React_tot(2); Pi_tot  = React_tot(3); 
EGTA_tot = React_tot(4); Lig_tot = React_tot(5); 

K_free  = Cation_free(1); Mg_free = Cation_free(2);
Ca_free = Cation_free(3); Na_free = Cation_free(4);
 
%% pK values corresponding to the dissociation constants of the associated reactions
% pK's of buffer reactions (pK's are mostly based on temperature 298.15 K and ionic 
% strength 0.1 M^0.5, and are not corrected for other temperature and ionic strength).
% MgHEGTA(-) = Mg(2+) + HEGTA(3-), pK_MgHEGTA = 3.37
% CaHEGTA(-) = Ca(2+) + HEGTA(3-), pK_CaHEGTA = 3.81

pK_HATP   = 6.48;           % HATP(3-)  = H(+) + ATP(4-)
pK_H2ATP  = 3.99;           % H2ATP(2-) = H(+) + HATP(3-)
pK_KATP   = 1.17;           % KATP(3-)  = K(+) + ATP(4-) 
pK_MgATP  = 4.19;           % MgATP(2-) = Mg(2+) + ATP(4-)
pK_MgHATP = 2.32;           % MgHATP(-) = Mg(2+) + HATP(3-)
pK_Mg2ATP = 1.70;           % Mg2ATP(0) = Mg(2+) + MgATP(2-)
pK_CaATP  = 3.86;           % CaATP(2-) = Ca(2+) + ATP(4-)
pK_CaHATP = 2.16;           % CaHATP(-) = Ca(2+) + HATP(3-)
pK_NaATP  = 1.31;           % NaATP(3-) = Na(+)  + ATP(4-)

pK_HADP   = 6.38;           % HADP(2-)  = H(+) + ADP(3-)
pK_H2ADP  = 3.87;           % H2ADP(-)  = H(+) + HADP(2-)
pK_KADP   = 1.0;            % KADP(2-)  = K(+) + ADP(3-)
pK_MgADP  = 3.23;           % MgADP(-)  = Mg(2+) + ADP(3-)
pK_MgHADP = 1.59;           % MgHADP(0) = Mg(2+) + HADP(2-)
pK_Mg2ADP = 1.0;            % Mg2ADP(+) = Mg(2+) + MgADP(-)
pK_CaADP  = 2.86;           % CaADP(-)  = Ca(2+) + ADP(3-)
pK_CaHADP = 1.48;           % CaHADP(0) = Ca(2+) + HADP(2-)
pK_NaADP  = 1.12;           % NaADP(2-) = Na(+)  + ADP(3-)

pK_H2PO4   = 6.7;           % H2PO4(-)   = H(+) + HPO4(2-)
pK_KHPO4   = 0.5;           % KHPO4(-)   = K(+) + HPO4(2-)
pK_KH2PO4  = 0.2;           % KH2PO4(0)  = K(+) + H2PO4(-)
pK_K2HPO4  = 0.5;           % K2HPO4(0)  = K(+) + KHPO4(-)
pK_MgHPO4  = 1.81;          % MgHPO4(0)  = Mg(2+) + HPO4(2-)
pK_MgH2PO4 = 1.19;          % MgH2PO4(+) = Mg(2+) + H2PO4(-)
pK_CaHPO4  = 1.6;           % CaHPO4(0)  = Ca(2+) + HPO4(2-)
pK_CaH2PO4 = 0.6;           % CaH2PO4(+) = Ca(2+) + H2PO4(-)
pK_NaHPO4  = 0.2;           % NaHPO4(-)  = Na(+) + HPO4(2-)

pK_HEGTA   = 9.58;          % HEGTA(3-)  = H(+) + EGTA(4-)
pK_H2EGTA  = 8.96;          % H2EGTA(2-) = H(+) + HEGTA(3-)
pK_KEGTA   = 1.22;          % KEGTA(3-)  = K(+) + EGTA(4-)
pK_MgEGTA  = 5.21;          % MgEGTA(2-) = Mg(2+) + EGTA(4-)
pK_MgHEGTA = 7.73;          % MgHEGTA(-) = H(+) + MgEGTA(2-)
pK_CaEGTA  = 10.97;         % CaEGTA(2-) = Ca(2+) + EGTA(4-)
pK_CaHEGTA = 3.81;          % CaHEGTA(-) = H(+) + CaEGTA(2-)
pK_NaEGTA  = 1.22;          % NaEGTA(3-) = Na(+) + EGTA(4-)

% These are set to make the buffering coefficients in the matrix appropriate 
pK_HLig  = -7.0;            % pK for proton buffering by proteins
pK_KLig  = -1.5;            % pK for pottasium buffering by proteins
pK_MgLig = 3.93;            % pK for magnesium buffering by proteins
pK_CaLig = 3.76;            % pK for calcium buffering by proteins
pK_NaLig = -0.5;            % pK for sodium buffering by proteins

%% Equilibrium relationships (based on given free cation concentrations). 
% Binding polynomials for reactants (ATP, ADP, Pi, EGTA, etc.)
P_ATP = 1 + 10^(-pH+pK_HATP) + 10^(-2*pH+pK_HATP+pK_H2ATP) + K_free*10^(pK_KATP) + ...
    Mg_free*10^(pK_MgATP) + Mg_free*10^(-pH+pK_HATP+pK_MgHATP) + ...
    Mg_free^2*10^(pK_MgATP+pK_Mg2ATP) + Ca_free*10^(pK_CaATP) + ...
    Ca_free*10^(-pH+pK_HATP+pK_CaHATP) + Na_free*10^(pK_NaATP);

P_ADP = 1 + 10^(-pH+pK_HADP) + 10^(-2*pH+pK_HADP+pK_H2ADP) + K_free*10^(pK_KADP) + ...
    Mg_free*10^(pK_MgADP) + Mg_free*10^(-pH+pK_HADP+pK_MgHADP) + ...
    Mg_free^2*10^(pK_MgADP+pK_Mg2ADP) + Ca_free*10^(pK_CaADP) + ...
    Ca_free*10^(-pH+pK_HADP+pK_CaHADP) + Na_free*10^(pK_NaADP);

P_Pi = 1 + 10^(-pH+pK_H2PO4) + K_free*10^(pK_KHPO4) + ...
    K_free*10^(-pH+pK_H2PO4+pK_KH2PO4) + K_free^2*10^(pK_K2HPO4+pK_KHPO4) + ...
    Mg_free*10^(pK_MgHPO4) + Mg_free*10^(-pH+pK_H2PO4+pK_MgH2PO4) + ...
    Ca_free*10^(pK_CaHPO4) + Ca_free*10^(-pH+pK_H2PO4+pK_CaH2PO4) + ...
    Na_free*10^(pK_NaHPO4);
P_EGTA = 1 + 10^(-pH+pK_HEGTA) + 10^(-2*pH+pK_HEGTA+pK_H2EGTA) + ...
    K_free*10^(pK_KEGTA) + Mg_free*10^(pK_MgEGTA) + ...
    Mg_free*10^(-pH+pK_MgHEGTA+pK_MgEGTA) + Ca_free*10^(pK_CaEGTA) + ...
    Ca_free*10^(-pH+pK_CaHEGTA+pK_CaEGTA)+Na_free*10^(pK_NaEGTA);
P_Lig = 1 + 10^(-pH+pK_HLig) + K_free*10^(pK_KLig) + Mg_free*10^(pK_MgLig)...
    + Ca_free*10^(pK_CaLig)+ Na_free*10^(pK_NaLig);

% Free ATP and cations-ATP bound species
ATP_free = ATP_tot/P_ATP;
HATP   = ATP_free*10^(-pH+pK_HATP);
H2ATP  = ATP_free*10^(-2*pH+pK_HATP+pK_H2ATP);
KATP   = ATP_free*K_free*10^(pK_KATP);
MgATP  = ATP_free*Mg_free*10^(pK_MgATP);
MgHATP = ATP_free*Mg_free*10^(-pH+pK_HATP+pK_MgHATP);
Mg2ATP = ATP_free*Mg_free^2*10^(pK_MgATP+pK_Mg2ATP);
CaATP  = ATP_free*Ca_free*10^(pK_CaATP);
CaHATP = ATP_free*Ca_free*10^(-pH+pK_HATP+pK_CaHATP);
NaATP  = ATP_free*Na_free*10^(pK_NaATP);

% Free ADP and cations-ADP bound species
ADP_free = ADP_tot/P_ADP;
HADP   = ADP_free*10^(-pH+pK_HADP);
H2ADP  = ADP_free*10^(-2*pH+pK_HADP+pK_H2ADP);
KADP   = ADP_free*K_free*10^(pK_KADP);
MgADP  = ADP_free*Mg_free*10^(pK_MgADP);
MgHADP = ADP_free*Mg_free*10^(-pH+pK_HADP+pK_MgHADP);
Mg2ADP = ADP_free*Mg_free^2*10^(pK_MgADP+pK_Mg2ADP);
CaADP  = ADP_free*Ca_free*10^(pK_CaADP);
CaHADP = ADP_free*Ca_free*10^(-pH+pK_HADP+pK_CaHADP);
NaADP  = ADP_free*Na_free*10^(pK_NaADP);

% Free Pi and cations-Pi bound species
Pi_free = Pi_tot/P_Pi;
HPO4    = Pi_free;         % reference species
H2PO4   = HPO4*10^(-pH+pK_H2PO4);
KHPO4   = HPO4*K_free*10^(pK_KHPO4);
KH2PO4  = HPO4*K_free*10^(-pH+pK_H2PO4+pK_KH2PO4);
K2HPO4  = HPO4*K_free^2*10^(pK_K2HPO4+pK_KHPO4);
MgHPO4  = HPO4*Mg_free*10^(pK_MgHPO4);
MgH2PO4 = HPO4*Mg_free*10^(-pH+pK_H2PO4+pK_MgH2PO4);
CaHPO4  = HPO4*Ca_free*10^(pK_CaHPO4);
CaH2PO4 = HPO4*Ca_free*10^(-pH+pK_H2PO4+pK_CaH2PO4);
NaHPO4  = HPO4*Na_free*10^(pK_NaHPO4);

% Free EGTA and cations-EGTA bound species
EGTA_free = EGTA_tot/P_EGTA;
HEGTA   = EGTA_free*10^(-pH+pK_HEGTA);
H2EGTA  = EGTA_free*10^(-2*pH+pK_HEGTA+pK_H2EGTA);
KEGTA   = EGTA_free*K_free*10^(pK_KEGTA);
MgEGTA  = EGTA_free*Mg_free*10^(pK_MgEGTA);
MgHEGTA = EGTA_free*Mg_free*10^(-pH+pK_MgHEGTA+pK_MgEGTA);
CaEGTA  = EGTA_free*Ca_free*10^(pK_CaEGTA);
CaHEGTA = EGTA_free*Ca_free*10^(-pH+pK_CaHEGTA+pK_CaEGTA);
NaEGTA  = EGTA_free*Na_free*10^(pK_NaEGTA);

% Free Ligand and cations-Ligand bound species
Lig_free = Lig_tot/P_Lig;
HLig  = Lig_free*10^(-pH+pK_HLig);
KLig  = Lig_free*K_free*10^(pK_KLig);
MgLig = Lig_free*Mg_free*10^(pK_MgLig);
CaLig = Lig_free*Ca_free*10^(pK_CaLig);
NaLig = Lig_free*Na_free*10^(pK_NaLig);

%% Set the reactant binding polynomials to one variable (vector).
P_React = [P_ATP; P_ADP; P_Pi; P_EGTA; P_Lig];

%% Set the reactant binding fractions to one variable (matrix).
ATP_frac  = [ATP_free, HATP, H2ATP, KATP, MgATP, MgHATP, Mg2ATP, CaATP, CaHATP, NaATP];
ADP_frac  = [ADP_free, HADP, H2ADP, KADP, MgADP, MgHADP, Mg2ADP, CaADP, CaHADP, NaADP];
Pi_frac   = [HPO4, H2PO4, KHPO4, KH2PO4, K2HPO4, MgHPO4, MgH2PO4, CaHPO4, CaH2PO4, NaHPO4];
EGTA_frac = [EGTA_free, HEGTA, H2EGTA, KEGTA, MgEGTA, MgHEGTA, CaEGTA, CaHEGTA, NaEGTA, 0];
Lig_frac  = [Lig_free, HLig, KLig, MgLig, CaLig, NaLig, 0, 0, 0, 0];
React_frac = [ATP_frac; ADP_frac; Pi_frac; EGTA_frac; Lig_frac];

%% Set the cation binding fractions to one variable (matrix).
H_frac  = [10^-pH, HATP, 2*H2ATP, MgHATP, HADP, 2*H2ADP, MgHADP, H2PO4, KH2PO4, ...
          MgH2PO4, CaH2PO4, HEGTA, 2*H2EGTA, MgHEGTA, CaHEGTA, HLig];
K_frac  = [K_free, KATP, KADP, KHPO4, KH2PO4, 2*K2HPO4, KEGTA, KLig,zeros(1,8)];
Mg_frac = [Mg_free, MgATP, MgHATP, 2*Mg2ATP, MgADP, MgHADP, 2*Mg2ADP, MgHPO4, ...
          MgH2PO4, MgEGTA, MgHEGTA, MgLig, zeros(1,4)];
Ca_frac = [Ca_free, CaATP, CaHATP, CaADP, CaHADP, CaHPO4, CaH2PO4, CaEGTA, ...
          CaHEGTA, CaLig, zeros(1,6)];
Na_frac = [Na_free, NaATP, NaADP, NaHPO4, NaEGTA, NaLig, zeros(1,10)];
Cation_frac = [H_frac; K_frac; Mg_frac; Ca_frac; Na_frac]; 
