function [P_React, React_frac, Cation_frac, ReactCation_pd] = ...
    get_binding_poly_frac_pd(React_tot, Cation_free, pH)

% This code computes the binding fractions of reactants (ATP,ADP,Pi,EGTA, etc.) 
% with cations (H+,K+,Mg2+,Ca2+,Na+). This code also computes partial derivatives 
% (pd) of cation-reactant-bound species with respect to reactants and cations. 
% The inputs are total reactants and free cations. The outputs are the binding 
% polynomials, binding fractions, and partial derivatives.

ATP_tot  = React_tot(1); ADP_tot = React_tot(2); Pi_tot  = React_tot(3); 
EGTA_tot = React_tot(4); Lig_tot = React_tot(5); 

K_free  = Cation_free(1); Mg_free = Cation_free(2);
Ca_free = Cation_free(3); Na_free = Cation_free(4);
 
%% pK values corresponding to the dissociation constants of the associated reactions
% pK's of buffer reactions (pK's are mostly based on temperature 298.15 K and ionic 
% strength 0.1 M^0.5, and are not corrected for other temperature and ionic strength).
% MgHEGTA(-) = Mg(2+) + HEGTA(3-), pK_MgHEGTA = 3.37
% CaHEGTA(-) = Ca(2+) + HEGTA(3-), pK_CaHEGTA = 3.81

pK_HATP   = 6.48;           % HATP(3-)  = H(+) + ATP(4-)
pK_H2ATP  = 3.99;           % H2ATP(2-) = H(+) + HATP(3-)
pK_KATP   = 1.17;           % KATP(3-)  = K(+) + ATP(4-)
pK_MgATP  = 4.19;           % MgATP(2-) = Mg(2+) + ATP(4-)
pK_MgHATP = 2.32;           % MgHATP(-) = Mg(2+) + HATP(3-)
pK_Mg2ATP = 1.70;           % Mg2ATP(0) = Mg(2+) + MgATP(2-) 
pK_CaATP  = 3.86;           % CaATP(2-) = Ca(2+) + ATP(4-)
pK_CaHATP = 2.16;           % CaHATP(-) = Ca(2+) + HATP(3-)
pK_NaATP  = 1.31;           % NaATP(3-) = Na(+)  + ATP(4-)

pK_HADP   = 6.38;           % HADP(2-)  = H(+) + ADP(3-)
pK_H2ADP  = 3.87;           % H2ADP(-)  = H(+) + HADP(2-)
pK_KADP   = 1.0;            % KADP(2-)  = K(+) + ADP(3-)
pK_MgADP  = 3.23;           % MgADP(-)  = Mg(2+) + ADP(3-)
pK_MgHADP = 1.59;           % MgHADP(0) = Mg(2+) + HADP(2-)
pK_Mg2ADP = 1.0;            % Mg2ADP(+) = Mg(2+) + MgADP(-)
pK_CaADP  = 2.86;           % CaADP(-)  = Ca(2+) + ADP(3-)
pK_CaHADP = 1.48;           % CaHADP(0) = Ca(2+) + HADP(2-)
pK_NaADP  = 1.12;           % NaADP(2-) = Na(+)  + ADP(3-)

pK_H2PO4   = 6.7;           % H2PO4(-)   = H(+) + HPO4(2-)
pK_KHPO4   = 0.5;           % KHPO4(-)   = K(+) + HPO4(2-)
pK_KH2PO4  = 0.2;           % KH2PO4(0)  = K(+) + H2PO4(-)
pK_K2HPO4  = 0.5;           % K2HPO4(0)  = K(+) + KHPO4(-)
pK_MgHPO4  = 1.81;          % MgHPO4(0)  = Mg(2+) + HPO4(2-)
pK_MgH2PO4 = 1.19;          % MgH2PO4(+) = Mg(2+) + H2PO4(-)
pK_CaHPO4  = 1.6;           % CaHPO4(0)  = Ca(2+) + HPO4(2-)
pK_CaH2PO4 = 0.6;           % CaH2PO4(+) = Ca(2+) + H2PO4(-)
pK_NaHPO4  = 0.2;           % NaHPO4(-)  = Na(+) + HPO4(2-)

pK_HEGTA   = 9.58;          % HEGTA(3-)  = H(+) + EGTA(4-)
pK_H2EGTA  = 8.96;          % H2EGTA(2-) = H(+) + HEGTA(3-)
pK_KEGTA   = 1.22;          % KEGTA(3-)  = K(+) + EGTA(4-)
pK_MgEGTA  = 5.21;          % MgEGTA(2-) = Mg(2+) + EGTA(4-)
pK_MgHEGTA = 7.73;          % MgHEGTA(-) = H(+) + MgEGTA(2-)
pK_CaEGTA  = 10.97;         % CaEGTA(2-) = Ca(2+) + EGTA(4-)
pK_CaHEGTA = 3.81;          % CaHEGTA(-) = H(+) + CaEGTA(2-)
pK_NaEGTA  = 1.22;          % NaEGTA(3-) = Na(+) + EGTA(4-)

% These are set to make the buffering coefficients in the matrix appropriate 
pK_HLig  = -7.0;            % pK for proton buffering by proteins
pK_KLig  = -1.5;            % pK for pottasium buffering by proteins
pK_MgLig = 3.93;            % pK for magnesium buffering by proteins
pK_CaLig = 3.76;            % pK for calcium buffering by proteins
pK_NaLig = -0.5;            % pK for sodium buffering by proteins

%% Equilibrium relationships (based on given free cation concentrations). 
% Partial derivatives of reactant binding polynomials w.r.t. cations
% Binding polynomial for ATP and its partial derivatives w.r.t. cations
P_ATP = 1 + 10^(-pH+pK_HATP) + 10^(-2*pH+pK_HATP+pK_H2ATP) + K_free*10^(pK_KATP) + ...
    Mg_free*10^(pK_MgATP) + Mg_free*10^(-pH+pK_HATP+pK_MgHATP) + ...
    Mg_free^2*10^(pK_MgATP+pK_Mg2ATP) + Ca_free*10^(pK_CaATP) + ...
    Ca_free*10^(-pH+pK_HATP+pK_CaHATP) + Na_free*10^(pK_NaATP);
P_ATP_H = 10^(pK_HATP) + 2*10^(-pH+pK_HATP+pK_H2ATP) + ...
    Mg_free*10^(pK_HATP+pK_MgHATP) + Ca_free*10^(pK_HATP+pK_CaHATP);
P_ATP_K = 10^(pK_KATP);
P_ATP_Mg = 10^(pK_MgATP) + 10^(-pH+pK_HATP+pK_MgHATP) + ...
    2*Mg_free*10^(pK_MgATP+pK_Mg2ATP);
P_ATP_Ca = 10^(pK_CaATP) + 10^(-pH+pK_HATP+pK_CaHATP);
P_ATP_Na = 10^(pK_NaATP); 

% Binding polynomial for ADP and its partial derivatives w.r.t. cations
P_ADP = 1 + 10^(-pH+pK_HADP) + 10^(-2*pH+pK_HADP+pK_H2ADP) + K_free*10^(pK_KADP) + ...
    Mg_free*10^(pK_MgADP) + Mg_free*10^(-pH+pK_HADP+pK_MgHADP) + ...
    Mg_free^2*10^(pK_MgADP+pK_Mg2ADP) + Ca_free*10^(pK_CaADP) + ...
    Ca_free*10^(-pH+pK_HADP+pK_CaHADP) + Na_free*10^(pK_NaADP);
P_ADP_H = 10^(pK_HADP) + 2*10^(-pH+pK_HADP+pK_H2ADP) + ...
    Mg_free*10^(pK_HADP+pK_MgHADP) + Ca_free*10^(pK_HADP+pK_CaHADP);
P_ADP_K = 10^(pK_KADP);
P_ADP_Mg = 10^(pK_MgADP) + 10^(-pH+pK_HADP+pK_MgHADP) + ...
    2*Mg_free*10^(pK_MgADP+pK_Mg2ADP);
P_ADP_Ca = 10^(pK_CaADP) + 10^(-pH+pK_HADP+pK_CaHADP);
P_ADP_Na = 10^(pK_NaADP);

% Binding polynomial for Pi and its partial derivatives w.r.t. cations
P_Pi = 1 + 10^(-pH+pK_H2PO4) + K_free*10^(pK_KHPO4) + ...
    K_free*10^(-pH+pK_H2PO4+pK_KH2PO4) + K_free^2*10^(pK_K2HPO4+pK_KHPO4) + ...
    Mg_free*10^(pK_MgHPO4) + Mg_free*10^(-pH+pK_H2PO4+pK_MgH2PO4) + ...
    Ca_free*10^(pK_CaHPO4) + Ca_free*10^(-pH+pK_H2PO4+pK_CaH2PO4) + ...
    Na_free*10^(pK_NaHPO4);
P_Pi_H = 10^(pK_H2PO4) + K_free*10^(pK_H2PO4+pK_KH2PO4) + ...
    Mg_free*10^(pK_H2PO4+pK_MgH2PO4) + Ca_free*10^(pK_H2PO4+pK_CaH2PO4);
P_Pi_K = 10^(pK_KHPO4) + 10^(-pH+pK_H2PO4+pK_KH2PO4) + ...
    2*K_free*10^(pK_K2HPO4+pK_KHPO4);
P_Pi_Mg = 10^(pK_MgHPO4) + 10^(-pH+pK_H2PO4+pK_MgH2PO4);
P_Pi_Ca = 10^(pK_CaHPO4) + 10^(-pH+pK_H2PO4+pK_CaH2PO4);
P_Pi_Na = 10^(pK_NaHPO4);

% Binding polynomial for EGTA and its partial derivatives w.r.t. cations
P_EGTA = 1 + 10^(-pH+pK_HEGTA) + 10^(-2*pH+pK_HEGTA+pK_H2EGTA) + ...
    K_free*10^(pK_KEGTA) + Mg_free*10^(pK_MgEGTA) + ...
    Mg_free*10^(-pH+pK_MgHEGTA+pK_MgEGTA) + Ca_free*10^(pK_CaEGTA) + ...
    Ca_free*10^(-pH+pK_CaHEGTA+pK_CaEGTA)+Na_free*10^(pK_NaEGTA);
P_EGTA_H = 10^(pK_HEGTA) + 2*10^(-pH+pK_HEGTA+pK_H2EGTA) + ...
    Mg_free*10^(pK_MgHEGTA+pK_MgEGTA) + Ca_free*10^(pK_CaHEGTA+pK_CaEGTA);
P_EGTA_K = 10^(pK_KEGTA);
P_EGTA_Mg = 10^(pK_MgEGTA) + 10^(-pH+pK_MgHEGTA+pK_MgEGTA);
P_EGTA_Ca = 10^(pK_CaEGTA) + 10^(-pH+pK_CaHEGTA+pK_CaEGTA);
P_EGTA_Na = 10^(pK_NaEGTA);

% Binding polynomial for ligand (protein) and its partial derivatives w.r.t. cations
P_Lig = 1 + 10^(-pH+pK_HLig) + K_free*10^(pK_KLig) + Mg_free*10^(pK_MgLig)...
    + Ca_free*10^(pK_CaLig)+ Na_free*10^(pK_NaLig);
P_Lig_H = 10^(pK_HLig);
P_Lig_K = 10^(pK_KLig);
P_Lig_Mg = 10^(pK_MgLig);
P_Lig_Ca = 10^(pK_CaLig);
P_Lig_Na = 10^(pK_NaLig);

%% Partial derivatives of ATP binding fractions w.r.t. cations and ATP
% Free ATP and its partial derivatives w.r.t. cations and ATP
ATP_free = ATP_tot/P_ATP;
ATP_free_H  = -ATP_tot*(P_ATP_H/P_ATP^2);
ATP_free_K  = -ATP_tot*(P_ATP_K/P_ATP^2);
ATP_free_Mg = -ATP_tot*(P_ATP_Mg/P_ATP^2);
ATP_free_Ca = -ATP_tot*(P_ATP_Ca/P_ATP^2);
ATP_free_Na = -ATP_tot*(P_ATP_Na/P_ATP^2);
ATP_free_ATP = 1/P_ATP;

% HATP and its partial derivatives w.r.t. cations and ATP
HATP = ATP_free*10^(-pH+pK_HATP);
HATP_H   = ATP_free_H*10^(-pH+pK_HATP) + ATP_free*10^(pK_HATP);
HATP_K   = ATP_free_K*10^(-pH+pK_HATP);
HATP_Mg  = ATP_free_Mg*10^(-pH+pK_HATP);
HATP_Ca  = ATP_free_Ca*10^(-pH+pK_HATP);
HATP_Na  = ATP_free_Na*10^(-pH+pK_HATP);
HATP_ATP = ATP_free_ATP*10^(-pH+pK_HATP);

% H2ATP and its partial derivatives w.r.t. cations and ATP
H2ATP = ATP_free*10^(-2*pH+pK_HATP+pK_H2ATP);
H2ATP_H = ATP_free_H*10^(-2*pH+pK_HATP+pK_H2ATP) + ...
    2*ATP_free*10^(-pH+pK_HATP+pK_H2ATP);
H2ATP_K   = ATP_free_K*10^(-2*pH+pK_HATP+pK_H2ATP);
H2ATP_Mg  = ATP_free_Mg*10^(-2*pH+pK_HATP+pK_H2ATP);
H2ATP_Ca  = ATP_free_Ca*10^(-2*pH+pK_HATP+pK_H2ATP);
H2ATP_Na = ATP_free_Na*10^(-2*pH+pK_HATP+pK_H2ATP);
H2ATP_ATP = ATP_free_ATP*10^(-2*pH+pK_HATP+pK_H2ATP);

% KATP and its partial derivatives w.r.t.cations and ATP
KATP = ATP_free*K_free*10^(pK_KATP);
KATP_H = ATP_free_H*K_free*10^(pK_KATP);
KATP_K = ATP_free_K*K_free*10^(pK_KATP) + ATP_free*10^(pK_KATP);
KATP_Mg = ATP_free_Mg*K_free*10^(pK_KATP);
KATP_Ca = ATP_free_Ca*K_free*10^(pK_KATP);
KATP_Na = ATP_free_Na*K_free*10^(pK_KATP);
KATP_ATP = ATP_free_ATP*K_free*10^(pK_KATP);

% MgATP and its partial derivatives w.r.t. cations and ATP
MgATP = ATP_free*Mg_free*10^(pK_MgATP);
MgATP_H = ATP_free_H*Mg_free*10^(pK_MgATP);
MgATP_K = ATP_free_K*Mg_free*10^(pK_MgATP);
MgATP_Mg = ATP_free_Mg*Mg_free*10^(pK_MgATP) + ATP_free*10^(pK_MgATP);
MgATP_Ca = ATP_free_Ca*Mg_free*10^(pK_MgATP);
MgATP_Na = ATP_free_Na*Mg_free*10^(pK_MgATP);
MgATP_ATP = ATP_free_ATP*Mg_free*10^(pK_MgATP);

% MgHATP and its partial derivatives w.r.t. cations and ATP
MgHATP = ATP_free*Mg_free*10^(-pH+pK_HATP+pK_MgHATP);
MgHATP_H = ATP_free_H*Mg_free*10^(-pH+pK_HATP+pK_MgHATP) + ...
    ATP_free*Mg_free*10^(pK_HATP+pK_MgHATP);
MgHATP_K = ATP_free_K*Mg_free*10^(-pH+pK_HATP+pK_MgHATP);
MgHATP_Mg = ATP_free_Mg*Mg_free*10^(-pH+pK_HATP+pK_MgHATP) + ...
    ATP_free*10^(-pH+pK_HATP+pK_MgHATP);
MgHATP_Ca = ATP_free_Ca*Mg_free*10^(-pH+pK_HATP+pK_MgHATP);
MgHATP_Na = ATP_free_Na*Mg_free*10^(-pH+pK_HATP+pK_MgHATP);
MgHATP_ATP = ATP_free_ATP*Mg_free*10^(-pH+pK_HATP+pK_MgHATP);

% Mg2ATP and its partial derivatives w.r.t. cations and ATP
Mg2ATP = ATP_free*Mg_free^2*10^(pK_MgATP+pK_Mg2ATP);
Mg2ATP_H = ATP_free_H*Mg_free^2*10^(pK_MgATP+pK_Mg2ATP);
Mg2ATP_K = ATP_free_K*Mg_free^2*10^(pK_MgATP+pK_Mg2ATP);
Mg2ATP_Mg = ATP_free_Mg*Mg_free^2*10^(pK_MgATP+pK_Mg2ATP) + ...
    2*ATP_free*Mg_free*10^(pK_MgATP+pK_Mg2ATP);
Mg2ATP_Ca = ATP_free_Ca*Mg_free^2*10^(pK_MgATP+pK_Mg2ATP);
Mg2ATP_Na = ATP_free_Na*Mg_free^2*10^(pK_MgATP+pK_Mg2ATP);
Mg2ATP_ATP = ATP_free_ATP*Mg_free^2*10^(pK_MgATP+pK_Mg2ATP);

% CaATP and its partial derivatives w.r.t. cations and ATP
CaATP = ATP_free*Ca_free*10^(pK_CaATP);
CaATP_H = ATP_free_H*Ca_free*10^(pK_CaATP);
CaATP_K = ATP_free_K*Ca_free*10^(pK_CaATP);
CaATP_Mg = ATP_free_Mg*Ca_free*10^(pK_CaATP);
CaATP_Ca = ATP_free_Ca*Ca_free*10^(pK_CaATP) + ATP_free*10^(pK_CaATP);
CaATP_Na = ATP_free_Na*Ca_free*10^(pK_CaATP);
CaATP_ATP = ATP_free_ATP*Ca_free*10^(pK_CaATP);

% CaHATP and its partial derivatives w.r.t. cations and ATP
CaHATP = ATP_free*Ca_free*10^(-pH+pK_HATP+pK_CaHATP);
CaHATP_H = ATP_free_H*Ca_free*10^(-pH+pK_HATP+pK_CaHATP) + ...
    ATP_free*Ca_free*10^(pK_HATP+pK_CaHATP);
CaHATP_K = ATP_free_K*Ca_free*10^(-pH+pK_HATP+pK_CaHATP);
CaHATP_Mg = ATP_free_Mg*Ca_free*10^(-pH+pK_HATP+pK_CaHATP);
CaHATP_Ca = ATP_free_Ca*Ca_free*10^(-pH+pK_HATP+pK_CaHATP) + ...
    ATP_free*10^(-pH+pK_HATP+pK_CaHATP);
CaHATP_Na = ATP_free_Na*Ca_free*10^(-pH+pK_HATP+pK_CaHATP);
CaHATP_ATP = ATP_free_ATP*Ca_free*10^(-pH+pK_HATP+pK_CaHATP);

% NaATP and its partial derivatives w.r.t. cations and ATP
NaATP = ATP_free*Na_free*10^(pK_NaATP);
NaATP_H = ATP_free_H*Na_free*10^(pK_NaATP);
NaATP_K = ATP_free_K*Na_free*10^(pK_NaATP);
NaATP_Mg = ATP_free_Mg*Na_free*10^(pK_NaATP);
NaATP_Ca = ATP_free_Ca*Na_free*10^(pK_NaATP);
NaATP_Na = ATP_free_Na*Na_free*10^(pK_NaATP) + ATP_free*10^(pK_NaATP);
NaATP_ATP = ATP_free_ATP*Na_free*10^(pK_NaATP);

%% Partial derivatives of ADP binding fractions w.r.t. cations and ADP
% Free ADP and its partial derivatives w.r.t. cations and ADP
ADP_free = ADP_tot/P_ADP;
ADP_free_H = -ADP_tot*(P_ADP_H/P_ADP^2);
ADP_free_K = -ADP_tot*(P_ADP_K/P_ADP^2);
ADP_free_Mg = -ADP_tot*(P_ADP_Mg/P_ADP^2);
ADP_free_Ca = -ADP_tot*(P_ADP_Ca/P_ADP^2);
ADP_free_Na = -ADP_tot*(P_ADP_Na/P_ADP^2);
ADP_free_ADP = 1/P_ADP;

% HADP and its partial derivatives w.r.t. H+, K+, Mg2+, Ca2+,Na+ and ADP
HADP = ADP_free*10^(-pH+pK_HADP);
HADP_H = ADP_free_H*10^(-pH+pK_HADP) + ADP_free*10^(pK_HADP);
HADP_K = ADP_free_K*10^(-pH+pK_HADP);
HADP_Mg = ADP_free_Mg*10^(-pH+pK_HADP);
HADP_Ca = ADP_free_Ca*10^(-pH+pK_HADP);
HADP_Na = ADP_free_Na*10^(-pH+pK_HADP);
HADP_ADP = ADP_free_ADP*10^(-pH+pK_HADP);

% H2ADP and its partial derivatives w.r.t. cations and ADP
H2ADP = ADP_free*10^(-2*pH+pK_HADP+pK_H2ADP);
H2ADP_H = ADP_free_H*10^(-2*pH+pK_HADP+pK_H2ADP) + ...
    2*ADP_free*10^(-pH+pK_HADP+pK_H2ADP);
H2ADP_K = ADP_free_K*10^(-2*pH+pK_HADP+pK_H2ADP);
H2ADP_Mg = ADP_free_Mg*10^(-2*pH+pK_HADP+pK_H2ADP);
H2ADP_Ca = ADP_free_Ca*10^(-2*pH+pK_HADP+pK_H2ADP);
H2ADP_Na = ADP_free_Na*10^(-2*pH+pK_HADP+pK_H2ADP);
H2ADP_ADP = ADP_free_ADP*10^(-2*pH+pK_HADP+pK_H2ADP);

% KADP and its partial derivatives w.r.t. cations and ADP
KADP = ADP_free*K_free*10^(pK_KADP);
KADP_H = ADP_free_H*K_free*10^(pK_KADP);
KADP_K = ADP_free_K*K_free*10^(pK_KADP) + ADP_free*10^(pK_KADP);
KADP_Mg = ADP_free_Mg*K_free*10^(pK_KADP);
KADP_Ca = ADP_free_Ca*K_free*10^(pK_KADP);
KADP_Na = ADP_free_Na*K_free*10^(pK_KADP);
KADP_ADP = ADP_free_ADP*K_free*10^(pK_KADP);

% MgADP and its partial derivatives w.r.t. cations and ADP
MgADP = ADP_free*Mg_free*10^(pK_MgADP);
MgADP_H = ADP_free_H*Mg_free*10^(pK_MgADP);
MgADP_K = ADP_free_K*Mg_free*10^(pK_MgADP);
MgADP_Mg = ADP_free_Mg*Mg_free*10^(pK_MgADP) + ADP_free*10^(pK_MgADP);
MgADP_Ca = ADP_free_Ca*Mg_free*10^(pK_MgADP);
MgADP_Na = ADP_free_Na*Mg_free*10^(pK_MgADP);
MgADP_ADP = ADP_free_ADP*Mg_free*10^(pK_MgADP);

% MgHADP and its partial derivatives w.r.t. cations and ADP
MgHADP = ADP_free*Mg_free*10^(-pH+pK_HADP+pK_MgHADP);
MgHADP_H = ADP_free_H*Mg_free*10^(-pH+pK_HADP+pK_MgHADP) + ...
    ADP_free*Mg_free*10^(pK_HADP+pK_MgHADP);
MgHADP_K = ADP_free_K*Mg_free*10^(-pH+pK_HADP+pK_MgHADP);
MgHADP_Mg = ADP_free_Mg*Mg_free*10^(-pH+pK_HADP+pK_MgHADP) + ...
    ADP_free*10^(-pH+pK_HADP+pK_MgHADP);
MgHADP_Ca = ADP_free_Ca*Mg_free*10^(-pH+pK_HADP+pK_MgHADP);
MgHADP_Na = ADP_free_Na*Mg_free*10^(-pH+pK_HADP+pK_MgHADP);
MgHADP_ADP = ADP_free_ADP*Mg_free*10^(-pH+pK_HADP+pK_MgHADP);

% Mg2ADP and its partial derivatives w.r.t. cations and ADP
Mg2ADP = ADP_free*Mg_free^2*10^(pK_MgADP+pK_Mg2ADP);
Mg2ADP_H = ADP_free_H*Mg_free^2*10^(pK_MgADP+pK_Mg2ADP);
Mg2ADP_K = ADP_free_K*Mg_free^2*10^(pK_MgADP+pK_Mg2ADP);
Mg2ADP_Mg = ADP_free_Mg*Mg_free^2*10^(pK_MgADP+pK_Mg2ADP) + ...
    2*ADP_free*Mg_free*10^(pK_MgADP+pK_Mg2ADP);
Mg2ADP_Ca = ADP_free_Ca*Mg_free^2*10^(pK_MgADP+pK_Mg2ADP);
Mg2ADP_Na = ADP_free_Na*Mg_free^2*10^(pK_MgADP+pK_Mg2ADP);
Mg2ADP_ADP = ADP_free_ADP*Mg_free^2*10^(pK_MgADP+pK_Mg2ADP);

% CaADP and its partial derivatives w.r.t. cations and ADP
CaADP = ADP_free*Ca_free*10^(pK_CaADP);
CaADP_H = ADP_free_H*Ca_free*10^(pK_CaADP);
CaADP_K = ADP_free_K*Ca_free*10^(pK_CaADP);
CaADP_Mg = ADP_free_Mg*Ca_free*10^(pK_CaADP);
CaADP_Ca = ADP_free_Ca*Ca_free*10^(pK_CaADP) + ADP_free*10^(pK_CaADP);
CaADP_Na = ADP_free_Na*Ca_free*10^(pK_CaADP);
CaADP_ADP = ADP_free_ADP*Ca_free*10^(pK_CaADP);

% CaHADP and its partial derivatives w.r.t. cations and ADP
CaHADP = ADP_free*Ca_free*10^(-pH+pK_HADP+pK_CaHADP);
CaHADP_H = ADP_free_H*Ca_free*10^(-pH+pK_HADP+pK_CaHADP) + ...
    ADP_free*Ca_free*10^(pK_HADP+pK_CaHADP);
CaHADP_K = ADP_free_K*Ca_free*10^(-pH+pK_HADP+pK_CaHADP);
CaHADP_Mg = ADP_free_Mg*Ca_free*10^(-pH+pK_HADP+pK_CaHADP);
CaHADP_Ca = ADP_free_Ca*Ca_free*10^(-pH+pK_HADP+pK_CaHADP) + ...
    ADP_free*10^(-pH+pK_HADP+pK_CaHADP);
CaHADP_Na = ADP_free_Na*Ca_free*10^(-pH+pK_HADP+pK_CaHADP);
CaHADP_ADP = ADP_free_ADP*Ca_free*10^(-pH+pK_HADP+pK_CaHADP);

% NaADP and its partial derivatives w.r.t. cations and ADP
NaADP = ADP_free*Na_free*10^(pK_NaADP);
NaADP_H = ADP_free_H*Na_free*10^(pK_NaADP);
NaADP_K = ADP_free_K*Na_free*10^(pK_NaADP); 
NaADP_Mg = ADP_free_Mg*Na_free*10^(pK_NaADP);
NaADP_Ca = ADP_free_Ca*Na_free*10^(pK_NaADP);
NaADP_Na = ADP_free_Na*Na_free*10^(pK_NaADP)+ ADP_free*10^(pK_NaADP);
NaADP_ADP = ADP_free_ADP*Na_free*10^(pK_NaADP);

%% Partial derivatives of Pi binding fractions w.r.t. cations and Pi
% Free Pi and its partial derivatives w.r.t. cations and Pi
Pi_free = Pi_tot/P_Pi;
Pi_free_H = -Pi_tot*(P_Pi_H/P_Pi^2);
Pi_free_K = -Pi_tot*(P_Pi_K/P_Pi^2);
Pi_free_Mg= -Pi_tot*(P_Pi_Mg/P_Pi^2);
Pi_free_Ca= -Pi_tot*(P_Pi_Ca/P_Pi^2);
Pi_free_Na = -Pi_tot*(P_Pi_Na/P_Pi^2);
Pi_free_Pi= 1/P_Pi;

% HPO4 and its partial derivatives w.r.t. cations and Pi
HPO4 = Pi_free;         % reference species
HPO4_H  = Pi_free_H;    % reference species
HPO4_K  = Pi_free_K;    % reference species
HPO4_Mg = Pi_free_Mg;   % reference species
HPO4_Ca = Pi_free_Ca;   % reference species
HPO4_Na = Pi_free_Na;   % reference species
HPO4_Pi = Pi_free_Pi;   % reference species

% H2PO4 and its partial derivatives w.r.t. cations and Pi
H2PO4 = HPO4*10^(-pH+pK_H2PO4);
H2PO4_H  = HPO4_H*10^(-pH+pK_H2PO4) + HPO4*10^(pK_H2PO4);
H2PO4_K  = HPO4_K*10^(-pH+pK_H2PO4);
H2PO4_Mg = HPO4_Mg*10^(-pH+pK_H2PO4);
H2PO4_Ca = HPO4_Ca*10^(-pH+pK_H2PO4);
H2PO4_Na = HPO4_Na*10^(-pH+pK_H2PO4);
H2PO4_Pi = HPO4_Pi*10^(-pH+pK_H2PO4);

% KHPO4 and its partial derivatives w.r.t. cations and Pi
KHPO4 = HPO4*K_free*10^(pK_KHPO4);
KHPO4_H  = HPO4_H*K_free*10^(pK_KHPO4);
KHPO4_K  = HPO4_K*K_free*10^(pK_KHPO4) + HPO4*10^(pK_KHPO4);
KHPO4_Mg = HPO4_Mg*K_free*10^(pK_KHPO4);
KHPO4_Ca = HPO4_Ca*K_free*10^(pK_KHPO4);
KHPO4_Na = HPO4_Na*K_free*10^(pK_KHPO4);
KHPO4_Pi = HPO4_Pi*K_free*10^(pK_KHPO4);

% KH2PO4 and its partial derivatives w.r.t. cations and Pi
KH2PO4 = HPO4*K_free*10^(-pH+pK_H2PO4+pK_KH2PO4);
KH2PO4_H  = HPO4_H*K_free*10^(-pH+pK_H2PO4+pK_KH2PO4) + ...
    HPO4*K_free*10^(pK_H2PO4+pK_KH2PO4);
KH2PO4_K  = HPO4_K*K_free*10^(-pH+pK_H2PO4+pK_KH2PO4) + ...
    HPO4*10^(-pH+pK_H2PO4+pK_KH2PO4);
KH2PO4_Mg = HPO4_Mg*K_free*10^(-pH+pK_H2PO4+pK_KH2PO4);
KH2PO4_Ca = HPO4_Ca*K_free*10^(-pH+pK_H2PO4+pK_KH2PO4);
KH2PO4_Na = HPO4_Na*K_free*10^(-pH+pK_H2PO4+pK_KH2PO4);
KH2PO4_Pi = HPO4_Pi*K_free*10^(-pH+pK_H2PO4+pK_KH2PO4);

% K2HPO4 and its partial derivatives w.r.t. cations and Pi
K2HPO4 = HPO4*K_free^2*10^(pK_K2HPO4+pK_KHPO4);
K2HPO4_H  = HPO4_H*K_free^2*10^(pK_K2HPO4+pK_KHPO4);
K2HPO4_K  = HPO4_K*K_free^2*10^(pK_K2HPO4+pK_KHPO4) + ...
    2*HPO4*K_free*10^(pK_K2HPO4+pK_KHPO4);
K2HPO4_Mg = HPO4_Mg*K_free^2*10^(pK_K2HPO4+pK_KHPO4);
K2HPO4_Ca = HPO4_Ca*K_free^2*10^(pK_K2HPO4+pK_KHPO4);
K2HPO4_Na = HPO4_Na*K_free^2*10^(pK_K2HPO4+pK_KHPO4);
K2HPO4_Pi = HPO4_Pi*K_free^2*10^(pK_K2HPO4+pK_KHPO4);

% MgHPO4 and its partial derivatives w.r.t. cations and Pi
MgHPO4 = HPO4*Mg_free*10^(pK_MgHPO4);
MgHPO4_H  = HPO4_H*Mg_free*10^(pK_MgHPO4);
MgHPO4_K  = HPO4_K*Mg_free*10^(pK_MgHPO4);
MgHPO4_Mg = HPO4_Mg*Mg_free*10^(pK_MgHPO4) + HPO4*10^(pK_MgHPO4);
MgHPO4_Ca = HPO4_Ca*Mg_free*10^(pK_MgHPO4);
MgHPO4_Na = HPO4_Na*Mg_free*10^(pK_MgHPO4);
MgHPO4_Pi = HPO4_Pi*Mg_free*10^(pK_MgHPO4);

% MgH2PO4 and its partial derivatives w.r.t. cations and Pi
MgH2PO4 = HPO4*Mg_free*10^(-pH+pK_H2PO4+pK_MgH2PO4);
MgH2PO4_H = HPO4_H*Mg_free*10^(-pH+pK_H2PO4+pK_MgH2PO4) + ...
    HPO4*Mg_free*10^(pK_H2PO4+pK_MgH2PO4);
MgH2PO4_K = HPO4_K*Mg_free*10^(-pH+pK_H2PO4+pK_MgH2PO4);
MgH2PO4_Mg = HPO4_Mg*Mg_free*10^(-pH+pK_H2PO4+pK_MgH2PO4) + ...
    HPO4*10^(-pH+pK_H2PO4+pK_MgH2PO4);
MgH2PO4_Ca = HPO4_Ca*Mg_free*10^(-pH+pK_H2PO4+pK_MgH2PO4);
MgH2PO4_Na = HPO4_Na*Mg_free*10^(-pH+pK_H2PO4+pK_MgH2PO4);
MgH2PO4_Pi = HPO4_Pi*Mg_free*10^(-pH+pK_H2PO4+pK_MgH2PO4);

% CaHPO4 and its partial derivatives w.r.t. cations and Pi
CaHPO4 = HPO4*Ca_free*10^(pK_CaHPO4);
CaHPO4_H  = HPO4_H*Ca_free*10^(pK_CaHPO4);
CaHPO4_K  = HPO4_K*Ca_free*10^(pK_CaHPO4);
CaHPO4_Mg = HPO4_Mg*Ca_free*10^(pK_CaHPO4);
CaHPO4_Ca = HPO4_Ca*Ca_free*10^(pK_CaHPO4) + HPO4*10^(pK_CaHPO4);
CaHPO4_Na = HPO4_Na*Ca_free*10^(pK_CaHPO4);
CaHPO4_Pi = HPO4_Pi*Ca_free*10^(pK_CaHPO4);

% CaH2PO4 and its partial derivatives w.r.t. cations and Pi
CaH2PO4 = HPO4*Ca_free*10^(-pH+pK_H2PO4+pK_CaH2PO4);
CaH2PO4_H = HPO4_H*Ca_free*10^(-pH+pK_H2PO4+pK_CaH2PO4) + ...
    HPO4*Ca_free*10^(pK_H2PO4+pK_CaH2PO4);
CaH2PO4_K = HPO4_K*Ca_free*10^(-pH+pK_H2PO4+pK_CaH2PO4);
CaH2PO4_Mg = HPO4_Mg*Ca_free*10^(-pH+pK_H2PO4+pK_CaH2PO4);
CaH2PO4_Ca = HPO4_Ca*Ca_free*10^(-pH+pK_H2PO4+pK_CaH2PO4) + ...
    HPO4*10^(-pH+pK_H2PO4+pK_CaH2PO4);
CaH2PO4_Na = HPO4_Na*Ca_free*10^(-pH+pK_H2PO4+pK_CaH2PO4);
CaH2PO4_Pi = HPO4_Pi*Ca_free*10^(-pH+pK_H2PO4+pK_CaH2PO4);

% NaHPO4 and its partial derivatives w.r.t.cations and Pi
NaHPO4 = HPO4*Na_free*10^(pK_NaHPO4);
NaHPO4_H = HPO4_H*Na_free*10^(pK_NaHPO4);
NaHPO4_K = HPO4_K*Na_free*10^(pK_NaHPO4);
NaHPO4_Mg = HPO4_Mg*Na_free*10^(pK_NaHPO4);
NaHPO4_Ca = HPO4_Ca*Na_free*10^(pK_NaHPO4);
NaHPO4_Na = HPO4_Na*Na_free*10^(pK_NaHPO4) + HPO4*10^(pK_NaHPO4);
NaHPO4_Pi = HPO4_Pi*Na_free*10^(pK_NaHPO4);

%% Partial derivatives of EGTA binding fractions w.r.t. cations and EGTA
% Free EGTA and its partial derivatives w.r.t. cations and EGTA
EGTA_free = EGTA_tot/P_EGTA;
EGTA_free_H  = -EGTA_tot*(P_EGTA_H/P_EGTA^2);
EGTA_free_K  = -EGTA_tot*(P_EGTA_K/P_EGTA^2);
EGTA_free_Mg = -EGTA_tot*(P_EGTA_Mg/P_EGTA^2);
EGTA_free_Ca = -EGTA_tot*(P_EGTA_Ca/P_EGTA^2);
EGTA_free_Na = -EGTA_tot*(P_EGTA_Na/P_EGTA^2);
EGTA_free_EGTA = 1/P_EGTA;

% HEGTA and its partial derivatives w.r.t. cations and EGTA
HEGTA = EGTA_free*10^(-pH+pK_HEGTA);
HEGTA_H  = EGTA_free_H*10^(-pH+pK_HEGTA) + EGTA_free*10^(pK_HEGTA);
HEGTA_K  = EGTA_free_K*10^(-pH+pK_HEGTA);
HEGTA_Mg = EGTA_free_Mg*10^(-pH+pK_HEGTA);
HEGTA_Ca = EGTA_free_Ca*10^(-pH+pK_HEGTA);
HEGTA_Na = EGTA_free_Na*10^(-pH+pK_HEGTA);
HEGTA_EGTA = EGTA_free_EGTA*10^(-pH+pK_HEGTA);

% H2EGTA and its partial derivatives w.r.t. cations and EGTA
H2EGTA = EGTA_free*10^(-2*pH+pK_HEGTA+pK_H2EGTA);
H2EGTA_H  = EGTA_free_H*10^(-2*pH+pK_HEGTA+pK_H2EGTA) + ...
    2*EGTA_free*10^(-pH+pK_HEGTA+pK_H2EGTA);
H2EGTA_K  = EGTA_free_K*10^(-2*pH+pK_HEGTA+pK_H2EGTA);
H2EGTA_Mg = EGTA_free_Mg*10^(-2*pH+pK_HEGTA+pK_H2EGTA);
H2EGTA_Ca = EGTA_free_Ca*10^(-2*pH+pK_HEGTA+pK_H2EGTA);
H2EGTA_Na = EGTA_free_Na*10^(-2*pH+pK_HEGTA+pK_H2EGTA);
H2EGTA_EGTA = EGTA_free_EGTA*10^(-2*pH+pK_HEGTA+pK_H2EGTA);

% KEGTA and its partial derivatives w.r.t. cations and EGTA
KEGTA = EGTA_free*K_free*10^(pK_KEGTA);
KEGTA_H  = EGTA_free_H*K_free*10^(pK_KEGTA);
KEGTA_K  = EGTA_free_K*K_free*10^(pK_KEGTA) + EGTA_free*10^(pK_KEGTA);
KEGTA_Mg = EGTA_free_Mg*K_free*10^(pK_KEGTA);
KEGTA_Ca = EGTA_free_Ca*K_free*10^(pK_KEGTA);
KEGTA_Na = EGTA_free_Na*K_free*10^(pK_KEGTA);
KEGTA_EGTA = EGTA_free_EGTA*K_free*10^(pK_KEGTA);

% MgEGTA and its partial derivatives w.r.t.cations  and EGTA
MgEGTA = EGTA_free*Mg_free*10^(pK_MgEGTA);
MgEGTA_H  = EGTA_free_H*Mg_free*10^(pK_MgEGTA);
MgEGTA_K  = EGTA_free_K*Mg_free*10^(pK_MgEGTA);
MgEGTA_Mg = EGTA_free_Mg*Mg_free*10^(pK_MgEGTA) + EGTA_free*10^(pK_MgEGTA);
MgEGTA_Ca = EGTA_free_Ca*Mg_free*10^(pK_MgEGTA);
MgEGTA_Na = EGTA_free_Na*Mg_free*10^(pK_MgEGTA);
MgEGTA_EGTA = EGTA_free_EGTA*Mg_free*10^(pK_MgEGTA);

% MgHEGTA and its partial derivatives w.r.t. cations and EGTA
MgHEGTA = EGTA_free*Mg_free*10^(-pH+pK_MgHEGTA+pK_MgEGTA);
MgHEGTA_H  = EGTA_free_H*Mg_free*10^(-pH+pK_MgHEGTA+pK_MgEGTA) + ...
    EGTA_free*Mg_free*10^(pK_MgHEGTA+pK_MgEGTA);
MgHEGTA_K  = EGTA_free_K*Mg_free*10^(-pH+pK_MgHEGTA+pK_MgEGTA);
MgHEGTA_Mg = EGTA_free_Mg*Mg_free*10^(-pH+pK_MgHEGTA+pK_MgEGTA) + ...
    EGTA_free*10^(-pH+pK_MgHEGTA+pK_MgEGTA);
MgHEGTA_Ca = EGTA_free_Ca*Mg_free*10^(-pH+pK_MgHEGTA+pK_MgEGTA);
MgHEGTA_Na = EGTA_free_Na*Mg_free*10^(-pH+pK_MgHEGTA+pK_MgEGTA);
MgHEGTA_EGTA = EGTA_free_EGTA*Mg_free*10^(-pH+pK_MgHEGTA+pK_MgEGTA);

% CaEGTA and its partial derivatives w.r.t. cations and EGTA
CaEGTA = EGTA_free*Ca_free*10^(pK_CaEGTA);
CaEGTA_H  = EGTA_free_H*Ca_free*10^(pK_CaEGTA);
CaEGTA_K  = EGTA_free_K*Ca_free*10^(pK_CaEGTA);
CaEGTA_Mg = EGTA_free_Mg*Ca_free*10^(pK_CaEGTA);
CaEGTA_Ca = EGTA_free_Ca*Ca_free*10^(pK_CaEGTA) + EGTA_free*10^(pK_CaEGTA);
CaEGTA_Na = EGTA_free_Na*Ca_free*10^(pK_CaEGTA);
CaEGTA_EGTA = EGTA_free_EGTA*Ca_free*10^(pK_CaEGTA);

% CaHEGTA and its partial derivatives w.r.t. cations and EGTA
CaHEGTA = EGTA_free*Ca_free*10^(-pH+pK_CaHEGTA+pK_CaEGTA);
CaHEGTA_H  = EGTA_free_H*Ca_free*10^(-pH+pK_CaHEGTA+pK_CaEGTA) + ...
    EGTA_free*Ca_free*10^(-pH+pK_CaHEGTA+pK_CaEGTA);
CaHEGTA_K  = EGTA_free_K*Ca_free*10^(-pH+pK_CaHEGTA+pK_CaEGTA);
CaHEGTA_Mg = EGTA_free_Mg*Ca_free*10^(-pH+pK_CaHEGTA+pK_CaEGTA);
CaHEGTA_Ca = EGTA_free_Ca*Ca_free*10^(-pH+pK_CaHEGTA+pK_CaEGTA) + ...
    EGTA_free*10^(-pH+pK_CaHEGTA+pK_CaEGTA);
CaHEGTA_Na = EGTA_free_Na*Ca_free*10^(-pH+pK_CaHEGTA+pK_CaEGTA);
CaHEGTA_EGTA = EGTA_free_EGTA*Ca_free*10^(-pH+pK_CaHEGTA+pK_CaEGTA);

% NaEGTA and its partial derivatives w.r.t. cations and EGTA
NaEGTA = EGTA_free*Na_free*10^(pK_NaEGTA);
NaEGTA_H = EGTA_free_H*Na_free*10^(pK_NaEGTA);
NaEGTA_K = EGTA_free_K*Na_free*10^(pK_NaEGTA); 
NaEGTA_Mg = EGTA_free_Mg*Na_free*10^(pK_NaEGTA);
NaEGTA_Ca = EGTA_free_Ca*Na_free*10^(pK_NaEGTA);
NaEGTA_Na = EGTA_free_Na*Na_free*10^(pK_NaEGTA)+ EGTA_free*10^(pK_NaEGTA);
NaEGTA_EGTA = EGTA_free_EGTA*Na_free*10^(pK_NaEGTA);

%% Partial derivatives of ligand binding fractions w.r.t. cations and ligand
% Free ligand (protein) and its partial derivatives w.r.t. cations and ligand
Lig_free = Lig_tot/P_Lig;
Lig_free_H  = -Lig_tot*(P_Lig_H/P_Lig^2);
Lig_free_K  = -Lig_tot*(P_Lig_K/P_Lig^2);
Lig_free_Mg = -Lig_tot*(P_Lig_Mg/P_Lig^2);
Lig_free_Ca = -Lig_tot*(P_Lig_Ca/P_Lig^2);
Lig_free_Na = -Lig_tot*(P_Lig_Na/P_Lig^2);
Lig_free_Lig = 1/P_Lig;

% HLig and its partial derivatives w.r.t. cations and ligand
HLig = Lig_free*10^(-pH+pK_HLig);
HLig_H  = Lig_free_H*10^(-pH+pK_HLig) + Lig_free*10^(pK_HLig);
HLig_K  = Lig_free_K*10^(-pH+pK_HLig);
HLig_Mg = Lig_free_Mg*10^(-pH+pK_HLig);
HLig_Ca = Lig_free_Ca*10^(-pH+pK_HLig);
HLig_Na  = Lig_free_Na*10^(-pH+pK_HLig);
HLig_Lig = Lig_free_Lig*10^(-pH+pK_HLig);

% KLig and its partial derivatives w.r.t.cations and ligand
KLig = Lig_free*K_free*10^(pK_KLig);
KLig_H  = Lig_free_H*K_free*10^(pK_KLig);
KLig_K  = Lig_free_K*K_free*10^(pK_KLig) + Lig_free*10^(pK_KLig);
KLig_Mg = Lig_free_Mg*K_free*10^(pK_KLig);
KLig_Ca = Lig_free_Ca*K_free*10^(pK_KLig);
KLig_Na = Lig_free_Na*K_free*10^(pK_KLig);
KLig_Lig = Lig_free_Lig*K_free*10^(pK_KLig);

% MgLig and its partial derivatives w.r.t. cations  and ligand
MgLig = Lig_free*Mg_free*10^(pK_MgLig);
MgLig_H  = Lig_free_H*Mg_free*10^(pK_MgLig);
MgLig_K  = Lig_free_K*Mg_free*10^(pK_MgLig);
MgLig_Mg = Lig_free_Mg*Mg_free*10^(pK_MgLig) + Lig_free*10^(pK_MgLig);
MgLig_Ca = Lig_free_Ca*Mg_free*10^(pK_MgLig);
MgLig_Na = Lig_free_Na*Mg_free*10^(pK_MgLig);
MgLig_Lig = Lig_free_Lig*Mg_free*10^(pK_MgLig);

% CaLig and its partial derivatives w.r.t.cations and ligand
CaLig = Lig_free*Ca_free*10^(pK_CaLig);
CaLig_H  = Lig_free_H*Ca_free*10^(pK_CaLig);
CaLig_K  = Lig_free_K*Ca_free*10^(pK_CaLig);
CaLig_Mg = Lig_free_Mg*Ca_free*10^(pK_CaLig);
CaLig_Ca = Lig_free_Ca*Ca_free*10^(pK_CaLig) + Lig_free*10^(pK_CaLig);
CaLig_Na = Lig_free_Na*Ca_free*10^(pK_CaLig);
CaLig_Lig = Lig_free_Lig*Ca_free*10^(pK_CaLig);

% NaLig and its partial derivatives w.r.t. cations and ligand
NaLig = Lig_free*Na_free*10^(pK_NaLig);
NaLig_H  = Lig_free_H*Na_free*10^(pK_NaLig);
NaLig_K  = Lig_free_K*Na_free*10^(pK_NaLig);
NaLig_Mg = Lig_free_Mg*Na_free*10^(pK_NaLig);
NaLig_Ca = Lig_free_Ca*Na_free*10^(pK_NaLig);
NaLig_Na = Lig_free_Na*Na_free*10^(pK_NaLig)+ Lig_free*10^(pK_NaLig);
NaLig_Lig = Lig_free_Lig*Na_free*10^(pK_NaLig);

%% Partial derivatives of H-bound reactants w.r.t. cations and reactants
% H-bound and its partial derivatives w.r.t.cations and reactants
HReact = HATP + 2*H2ATP + MgHATP + HADP + 2*H2ADP + ...
    MgHADP + H2PO4 + KH2PO4 + MgH2PO4 + CaH2PO4 + HEGTA + ...
    2*H2EGTA + MgHEGTA + CaHEGTA + HLig;
HReact_H = HATP_H + 2*H2ATP_H + MgHATP_H + HADP_H + 2*H2ADP_H + ...
    MgHADP_H + H2PO4_H + KH2PO4_H + MgH2PO4_H + CaH2PO4_H + HEGTA_H + ...
    2*H2EGTA_H + MgHEGTA_H + CaHEGTA_H + HLig_H;
HReact_K = HATP_K + 2*H2ATP_K + MgHATP_K + HADP_K + 2*H2ADP_K + ...
    MgHADP_K + H2PO4_K + KH2PO4_K + MgH2PO4_K + CaH2PO4_K + HEGTA_K + ...
    2*H2EGTA_K + MgHEGTA_K + CaHEGTA_K + HLig_K;
HReact_Mg = HATP_Mg + 2*H2ATP_Mg + MgHATP_Mg + HADP_Mg + 2*H2ADP_Mg + ...
    MgHADP_Mg + H2PO4_Mg + KH2PO4_Mg + MgH2PO4_Mg + CaH2PO4_Mg + HEGTA_Mg + ...
    2*H2EGTA_Mg + MgHEGTA_Mg + CaHEGTA_Mg + HLig_Mg;
HReact_Ca = HATP_Ca + 2*H2ATP_Ca + MgHATP_Ca + HADP_Ca + 2*H2ADP_Ca + ...
    MgHADP_Ca + H2PO4_Ca + KH2PO4_Ca + MgH2PO4_Ca + CaH2PO4_Ca + HEGTA_Ca + ...
    2*H2EGTA_Ca + MgHEGTA_Ca + CaHEGTA_Ca + HLig_Ca;
HReact_Na = HATP_Na + 2*H2ATP_Na + MgHATP_Na + HADP_Na + 2*H2ADP_Na + ...
    MgHADP_Na + H2PO4_Na + KH2PO4_Na + MgH2PO4_Na + CaH2PO4_Na + HEGTA_Na + ...
    2*H2EGTA_Na + MgHEGTA_Na + CaHEGTA_Na + HLig_Na;
HReact_ATP = HATP_ATP + 2*H2ATP_ATP + MgHATP_ATP;
HReact_ADP = HADP_ADP + 2*H2ADP_ADP + MgHADP_ADP;
HReact_Pi = H2PO4_Pi + KH2PO4_Pi + MgH2PO4_Pi + CaH2PO4_Pi;
HReact_EGTA = HEGTA_EGTA + 2*H2EGTA_EGTA + MgHEGTA_EGTA + CaHEGTA_EGTA;
HReact_Lig = HLig_Lig;

%% Partial derivatives of K-bound reactants w.r.t. cations and reactants
% K-bound and its partial derivatives w.r.t. cations and reactants
KReact = KATP + KADP + KHPO4 + KH2PO4 + 2*K2HPO4 + KEGTA + KLig;
KReact_H = KATP_H + KADP_H + KHPO4_H + KH2PO4_H + 2*K2HPO4_H + KEGTA_H + ...
    KLig_H;
KReact_K = KATP_K + KADP_K + KHPO4_K + KH2PO4_K + 2*K2HPO4_K + KEGTA_K + ...
    KLig_K;
KReact_Mg = KATP_Mg + KADP_Mg + KHPO4_Mg + KH2PO4_Mg + 2*K2HPO4_Mg + ...
    KEGTA_Mg + KLig_Mg;
KReact_Ca = KATP_Ca + KADP_Ca + KHPO4_Ca + KH2PO4_Ca + 2*K2HPO4_Ca + ...
    KEGTA_Ca + KLig_Ca;
KReact_Na = KATP_Na + KADP_Na + KHPO4_Na + KH2PO4_Na + 2*K2HPO4_Na + ...
    KEGTA_Na + KLig_Na;
KReact_ATP = KATP_ATP;
KReact_ADP = KADP_ADP; 
KReact_Pi = KHPO4_Pi + KH2PO4_Pi + 2*K2HPO4_Pi;
KReact_EGTA = KEGTA_EGTA;
KReact_Lig = KLig_Lig;

%% Partial derivatives of Mg-bound reactants w.r.t. cations and reactants
% Mg-bound and its partial derivatives w.r.t. cations and reactants
MgReact = MgATP + MgHATP + 2*Mg2ATP + MgADP + MgHADP + ...
    2*Mg2ADP + MgHPO4 + MgH2PO4 + MgEGTA + MgHEGTA + MgLig ;
MgReact_H = MgATP_H + MgHATP_H + 2*Mg2ATP_H + MgADP_H + MgHADP_H + ...
    2*Mg2ADP_H + MgHPO4_H + MgH2PO4_H + MgEGTA_H + MgHEGTA_H + ...
    MgLig_H;
MgReact_K = MgATP_K + MgHATP_K + 2*Mg2ATP_K + MgADP_K + MgHADP_K + ...
    2*Mg2ADP_K + MgHPO4_K + MgH2PO4_K + MgEGTA_K + MgHEGTA_K + ...
    MgLig_K;
MgReact_Mg = MgATP_Mg + MgHATP_Mg + 2*Mg2ATP_Mg + MgADP_Mg + MgHADP_Mg + ...
    2*Mg2ADP_Mg + MgHPO4_Mg + MgH2PO4_Mg + MgEGTA_Mg + MgHEGTA_Mg + ...
    MgLig_Mg;
MgReact_Ca = MgATP_Ca + MgHATP_Ca + 2*Mg2ATP_Ca + MgADP_Ca + MgHADP_Ca + ...
    2*Mg2ADP_Ca + MgHPO4_Ca + MgH2PO4_Ca + MgEGTA_Ca + MgHEGTA_Ca + ...
    MgLig_Ca;
MgReact_Na = MgATP_Na + MgHATP_Na + 2*Mg2ATP_Na + MgADP_Na + MgHADP_Na + ...
    2*Mg2ADP_Na + MgHPO4_Na + MgH2PO4_Na + MgEGTA_Na + MgHEGTA_Na + ...
    MgLig_Na;
MgReact_ATP = MgATP_ATP + MgHATP_ATP + 2*Mg2ATP_ATP;
MgReact_ADP = MgADP_ADP + MgHADP_ADP + 2*Mg2ADP_ADP;
MgReact_Pi = MgHPO4_Pi + MgH2PO4_Pi;
MgReact_EGTA = MgEGTA_EGTA + MgHEGTA_EGTA;
MgReact_Lig = MgLig_Lig;

%% Partial derivatives of Ca-bound reactants w.r.t. cations and reactants
% Ca-bound and its partial derivatives w.r.t. cations,and reactants
CaReact = CaATP + CaHATP + CaADP + CaHADP + CaHPO4 + ...
    CaH2PO4 + CaEGTA + CaHEGTA + CaLig;
CaReact_H = CaATP_H + CaHATP_H + CaADP_H + CaHADP_H + CaHPO4_H + ...
    CaH2PO4_H + CaEGTA_H + CaHEGTA_H + CaLig_H;
CaReact_K = CaATP_K + CaHATP_K + CaADP_K + CaHADP_K + CaHPO4_K + ...
    CaH2PO4_K + CaEGTA_K + CaHEGTA_K + CaLig_K;
CaReact_Mg = CaATP_Mg + CaHATP_Mg + CaADP_Mg + CaHADP_Mg + CaHPO4_Mg + ...
    CaH2PO4_Mg + CaEGTA_Mg + CaHEGTA_Mg + CaLig_Mg;
CaReact_Ca = CaATP_Ca + CaHATP_Ca + CaADP_Ca + CaHADP_Ca + CaHPO4_Ca + ...
    CaH2PO4_Ca + CaEGTA_Ca + CaHEGTA_Ca + CaLig_Ca;
CaReact_Na = CaATP_Na + CaHATP_Na + CaADP_Na + CaHADP_Na + CaHPO4_Na + ...
    CaH2PO4_Na + CaEGTA_Na + CaHEGTA_Na + CaLig_Na;
CaReact_ATP = CaATP_ATP + CaHATP_ATP;
CaReact_ADP = CaADP_ADP + CaHADP_ADP;
CaReact_Pi  = CaHPO4_Pi + CaH2PO4_Pi;
CaReact_EGTA = CaEGTA_EGTA + CaHEGTA_EGTA;
CaReact_Lig = CaLig_Lig;

%% Partial derivatives of Na-bound reactants w.r.t. cations and reactants
% Na-bound and its partial derivatives w.r.t. cations, and reactants
NaReact = NaATP + NaADP + NaHPO4 + NaEGTA  + NaLig;
NaReact_H  = NaATP_H + NaADP_H + NaHPO4_H + NaEGTA_H + NaLig_H;
NaReact_K  = NaATP_K + NaADP_K + NaHPO4_K + NaEGTA_K + NaLig_K;
NaReact_Mg = NaATP_Mg + NaADP_Mg + NaHPO4_Mg + NaEGTA_Mg + NaLig_Mg;
NaReact_Ca = NaATP_Ca + NaADP_Ca + NaHPO4_Ca + NaEGTA_Ca + NaLig_Ca;
NaReact_Na = NaATP_Na + NaADP_Na + NaHPO4_Na + NaEGTA_Na + NaLig_Na;
NaReact_ATP= NaATP_ATP;
NaReact_ADP= NaADP_ADP; 
NaReact_Pi = NaHPO4_Pi;
NaReact_EGTA = NaEGTA_EGTA;
NaReact_Lig = NaLig_Lig;

%% Set the reactant binding polynomials to one variable (vector).
P_React = [P_ATP; P_ADP; P_Pi; P_EGTA; P_Lig];

%% Set the reactant binding fractions to one variable (matrix).
ATP_frac  = [ATP_free, HATP, H2ATP, KATP, MgATP, MgHATP, Mg2ATP, CaATP, CaHATP, NaATP];
ADP_frac  = [ADP_free, HADP, H2ADP, KADP, MgADP, MgHADP, Mg2ADP, CaADP, CaHADP, NaADP];
Pi_frac   = [HPO4, H2PO4, KHPO4, KH2PO4, K2HPO4, MgHPO4, MgH2PO4, CaHPO4, CaH2PO4, NaHPO4];
EGTA_frac = [EGTA_free, HEGTA, H2EGTA, KEGTA, MgEGTA, MgHEGTA, CaEGTA, CaHEGTA, NaEGTA, 0];
Lig_frac  = [Lig_free, HLig, KLig, MgLig, CaLig, NaLig, 0, 0, 0, 0];
React_frac = [ATP_frac; ADP_frac; Pi_frac; EGTA_frac; Lig_frac];

%% Set the cation binding fractions to one variable (matrix).
H_frac  = [10^-pH, HATP, 2*H2ATP, MgHATP, HADP, 2*H2ADP, MgHADP, H2PO4, KH2PO4, ...
          MgH2PO4, CaH2PO4, HEGTA, 2*H2EGTA, MgHEGTA, CaHEGTA, HLig];
K_frac  = [K_free, KATP, KADP, KHPO4, KH2PO4, 2*K2HPO4, KEGTA, KLig, zeros(1,8)];
Mg_frac = [Mg_free, MgATP, MgHATP, 2*Mg2ATP, MgADP, MgHADP, 2*Mg2ADP, MgHPO4, ...
          MgH2PO4, MgEGTA, MgHEGTA, MgLig, zeros(1,4)];
Ca_frac = [Ca_free, CaATP, CaHATP, CaADP, CaHADP, CaHPO4, CaH2PO4, CaEGTA, ...
          CaHEGTA, CaLig,zeros(1,6)];
Na_frac = [Na_free, NaATP, NaADP, NaHPO4, NaEGTA, NaLig,zeros(1,10)];
Cation_frac = [H_frac; K_frac; Mg_frac; Ca_frac; Na_frac];

%% Set the partial derivatives of reactact-bound-cations to one variable (matrix).
HReact_pd = [HReact_H, HReact_K, HReact_Mg, HReact_Ca, HReact_Na, HReact_ATP, ...
    HReact_ADP, HReact_Pi, HReact_EGTA, HReact_Lig];
KReact_pd = [KReact_H, KReact_K, KReact_Mg, KReact_Ca, KReact_Na, KReact_ATP, ...
    KReact_ADP, KReact_Pi, KReact_EGTA, KReact_Lig];
MgReact_pd = [MgReact_H, MgReact_K, MgReact_Mg, MgReact_Ca,MgReact_Na, MgReact_ATP, ...
    MgReact_ADP, MgReact_Pi, MgReact_EGTA, MgReact_Lig];
CaReact_pd = [CaReact_H, CaReact_K, CaReact_Mg, CaReact_Ca,CaReact_Na, CaReact_ATP, ...
    CaReact_ADP, CaReact_Pi, CaReact_EGTA, CaReact_Lig];
NaReact_pd  = [NaReact_H, NaReact_K, NaReact_Mg, NaReact_Ca, NaReact_Na, NaReact_ATP, ...
    NaReact_ADP, NaReact_Pi, NaReact_EGTA, NaReact_Lig];
ReactCation_pd = [HReact_pd; KReact_pd; MgReact_pd; CaReact_pd; NaReact_pd]; 

